@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2008-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2008-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -76,7 +76,7 @@ IDMEFCriteria::IDMEFCriteria()
 }
 
 
-IDMEFCriteria IDMEFCriteria::Clone()
+IDMEFCriteria IDMEFCriteria::clone() const
 {
         int ret;
         idmef_criteria_t *cl;
@@ -89,19 +89,19 @@ IDMEFCriteria IDMEFCriteria::Clone()
 }
 
 
-void IDMEFCriteria::ANDCriteria(const IDMEFCriteria &criteria)
+void IDMEFCriteria::andCriteria(const IDMEFCriteria &criteria)
 {
         idmef_criteria_and_criteria(this->_criteria, criteria._criteria);
 }
 
 
-void IDMEFCriteria::ORCriteria(const IDMEFCriteria &criteria)
+void IDMEFCriteria::orCriteria(const IDMEFCriteria &criteria)
 {
         idmef_criteria_or_criteria(this->_criteria, criteria._criteria);
 }
 
 
-int IDMEFCriteria::Match(IDMEF *message)
+int IDMEFCriteria::match(IDMEF *message) const
 {
         int ret;
 
@@ -113,7 +113,7 @@ int IDMEFCriteria::Match(IDMEF *message)
 }
 
 
-const std::string IDMEFCriteria::ToString() const
+const std::string IDMEFCriteria::toString() const
 {
         int ret;
         std::string s;
@@ -138,10 +138,18 @@ const std::string IDMEFCriteria::ToString() const
 
 IDMEFCriteria::operator const std::string() const
 {
-        return ToString();
+        return toString();
 }
 
 
+
+IDMEFCriteria::operator idmef_criteria_t *() const
+{
+        return _criteria;
+}
+
+
+
 IDMEFCriteria &IDMEFCriteria::operator=(const IDMEFCriteria &criteria)
 {
         if ( this != &criteria && _criteria != criteria._criteria ) {
