@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2008-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2008-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,16 +15,19 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
 #include "idmef.hxx"
 #include "idmef-path.hxx"
+#include "idmef-class.hxx"
 #include "prelude-error.hxx"
 
+#include "idmef-object-prv.h"
+
 using namespace Prelude;
 
 
@@ -38,6 +41,19 @@ IDMEFPath::IDMEFPath(const char *buffer)
 }
 
 
+
+IDMEFPath::IDMEFPath(IDMEF &idmef, const char *buffer)
+{
+        int ret;
+        idmef_object_t *obj = (idmef_object_t *) idmef;
+
+        ret = idmef_path_new_from_root_fast(&_path, obj->_idmef_object_id, buffer);
+        if ( ret < 0 )
+                throw PreludeError(ret);
+}
+
+
+
 IDMEFPath::IDMEFPath(idmef_path_t *path)
 {
         _path = path;
@@ -56,7 +72,7 @@ IDMEFPath::~IDMEFPath()
 }
 
 
-IDMEFValue IDMEFPath::Get(IDMEF &message)
+IDMEFValue IDMEFPath::get(const IDMEF &message) const
 {
         int ret;
         idmef_value_t *value;
@@ -73,20 +89,20 @@ IDMEFValue IDMEFPath::Get(IDMEF &message)
 
 
 
-idmef_value_type_id_t IDMEFPath::GetValueType(int depth)
+Prelude::IDMEFValue::IDMEFValueTypeEnum IDMEFPath::getValueType(int depth) const
 {
-        return idmef_path_get_value_type(_path, depth);
+        return (Prelude::IDMEFValue::IDMEFValueTypeEnum) idmef_path_get_value_type(_path, depth);
 }
 
 
-int IDMEFPath::CheckOperator(idmef_criterion_operator_t op)
+int IDMEFPath::checkOperator(idmef_criterion_operator_t op) const
 {
         return idmef_path_check_operator(_path, op);
 }
 
 
 
-idmef_criterion_operator_t IDMEFPath::GetApplicableOperators()
+idmef_criterion_operator_t IDMEFPath::getApplicableOperators() const
 {
         int ret;
         idmef_criterion_operator_t res;
@@ -99,7 +115,33 @@ idmef_criterion_operator_t IDMEFPath::GetApplicableOperators()
 }
 
 
-void IDMEFPath::Set(IDMEF &message, IDMEFValue *value)
+
+void IDMEFPath::set(IDMEF &message, const std::vector<IDMEF> &value) const
+{
+        int ret;
+        IDMEFValue v = value;
+
+        ret = idmef_path_set(_path, message, v);
+        if ( ret < 0 )
+                throw PreludeError(ret);
+}
+
+
+void IDMEFPath::set(IDMEF &message, IDMEF *value) const
+{
+        int ret;
+
+        if ( ! value )
+                ret = idmef_path_set(_path, message, NULL);
+        else
+                ret = idmef_path_set(_path, message, IDMEFValue(value));
+
+        if ( ret < 0 )
+                throw PreludeError(ret);
+}
+
+
+void IDMEFPath::set(IDMEF &message, IDMEFValue *value) const
 {
         int ret;
 
@@ -113,7 +155,7 @@ void IDMEFPath::Set(IDMEF &message, IDMEFValue *value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, std::vector<IDMEFValue> value)
+void IDMEFPath::set(IDMEF &message, const std::vector<IDMEFValue> &value) const
 {
         int ret;
         IDMEFValue v = value;
@@ -124,7 +166,7 @@ void IDMEFPath::Set(IDMEF &message, std::vector<IDMEFValue> value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, IDMEFValue &value)
+void IDMEFPath::set(IDMEF &message, IDMEFValue &value) const
 {
         int ret;
 
@@ -134,7 +176,7 @@ void IDMEFPath::Set(IDMEF &message, IDMEFValue &value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, std::string value)
+void IDMEFPath::set(IDMEF &message, const std::string &value) const
 {
         int ret;
 
@@ -144,7 +186,7 @@ void IDMEFPath::Set(IDMEF &message, std::string value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, const char *value)
+void IDMEFPath::set(IDMEF &message, const char *value) const
 {
         int ret;
 
@@ -159,7 +201,7 @@ void IDMEFPath::Set(IDMEF &message, const char *value)
 
 
 
-void IDMEFPath::Set(IDMEF &message, int8_t value)
+void IDMEFPath::set(IDMEF &message, int32_t value) const
 {
         int ret;
 
@@ -169,7 +211,8 @@ void IDMEFPath::Set(IDMEF &message, int8_t value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, uint8_t value)
+
+void IDMEFPath::set(IDMEF &message, int64_t value) const
 {
         int ret;
 
@@ -179,7 +222,7 @@ void IDMEFPath::Set(IDMEF &message, uint8_t value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, int16_t value)
+void IDMEFPath::set(IDMEF &message, uint64_t value) const
 {
         int ret;
 
@@ -189,16 +232,7 @@ void IDMEFPath::Set(IDMEF &message, int16_t value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, uint16_t value)
-{
-        int ret;
-
-        ret = idmef_path_set(_path, message, IDMEFValue(value));
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-void IDMEFPath::Set(IDMEF &message, int32_t value)
+void IDMEFPath::set(IDMEF &message, float value) const
 {
         int ret;
 
@@ -208,16 +242,7 @@ void IDMEFPath::Set(IDMEF &message, int32_t value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, uint32_t value)
-{
-        int ret;
-
-        ret = idmef_path_set(_path, message, IDMEFValue(value));
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-void IDMEFPath::Set(IDMEF &message, int64_t value)
+void IDMEFPath::set(IDMEF &message, double value) const
 {
         int ret;
 
@@ -227,37 +252,7 @@ void IDMEFPath::Set(IDMEF &message, int64_t value)
 }
 
 
-void IDMEFPath::Set(IDMEF &message, uint64_t value)
-{
-        int ret;
-
-        ret = idmef_path_set(_path, message, IDMEFValue(value));
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-
-void IDMEFPath::Set(IDMEF &message, float value)
-{
-        int ret;
-
-        ret = idmef_path_set(_path, message, IDMEFValue(value));
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-
-void IDMEFPath::Set(IDMEF &message, double value)
-{
-        int ret;
-
-        ret = idmef_path_set(_path, message, IDMEFValue(value));
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-
-void IDMEFPath::Set(IDMEF &message, IDMEFTime &time)
+void IDMEFPath::set(IDMEF &message, IDMEFTime &time) const
 {
         int ret;
 
@@ -268,53 +263,53 @@ void IDMEFPath::Set(IDMEF &message, IDMEFTime &time)
 
 
 
-idmef_class_id_t IDMEFPath::GetClass(int depth)
+IDMEFClass IDMEFPath::getClass(int depth) const
 {
-        return idmef_path_get_class(_path, depth);
+        return IDMEFClass(idmef_path_get_class(_path, depth));
 }
 
 
-int IDMEFPath::SetIndex(unsigned int index, int depth)
+int IDMEFPath::setIndex(unsigned int index, int depth)
 {
         if ( depth < 0 )
-                depth = GetDepth();
+                depth = getDepth();
 
         return idmef_path_set_index(_path, depth, index);
 }
 
 
-int IDMEFPath::UndefineIndex(int depth)
+int IDMEFPath::undefineIndex(int depth)
 {
         if ( depth < 0 )
-                depth = GetDepth();
+                depth = getDepth();
 
         return idmef_path_undefine_index(_path, depth);
 }
 
 
-int IDMEFPath::GetIndex(int depth)
+int IDMEFPath::getIndex(int depth) const
 {
         if ( depth < 0 )
-                depth = GetDepth();
+                depth = getDepth();
 
         return idmef_path_get_index(_path, depth);
 }
 
 
-int IDMEFPath::MakeChild(const char *child_name, unsigned int index=0)
+int IDMEFPath::makeChild(const char *child_name, unsigned int index=0)
 {
         return idmef_path_make_child(_path, child_name, index);
 }
 
 
 
-int IDMEFPath::MakeParent()
+int IDMEFPath::makeParent()
 {
         return idmef_path_make_parent(_path);
 }
 
 
-int IDMEFPath::Compare(IDMEFPath *path, int depth)
+int IDMEFPath::compare(IDMEFPath *path, int depth) const
 {
         int ret;
 
@@ -327,7 +322,7 @@ int IDMEFPath::Compare(IDMEFPath *path, int depth)
 }
 
 
-IDMEFPath IDMEFPath::Clone()
+IDMEFPath IDMEFPath::clone() const
 {
         int ret;
         idmef_path_t *cpath;
@@ -340,31 +335,31 @@ IDMEFPath IDMEFPath::Clone()
 }
 
 
-const char *IDMEFPath::GetName(int depth)
+const char *IDMEFPath::getName(int depth) const
 {
         return idmef_path_get_name(_path, depth);
 }
 
 
-bool IDMEFPath::IsAmbiguous()
+bool IDMEFPath::isAmbiguous() const
 {
         return idmef_path_is_ambiguous(_path);
 }
 
 
-int IDMEFPath::HasLists()
+int IDMEFPath::hasLists() const
 {
         return idmef_path_has_lists(_path);
 }
 
 
-bool IDMEFPath::IsList(int depth)
+bool IDMEFPath::isList(int depth) const
 {
         return idmef_path_is_list(_path, depth);
 }
 
 
-unsigned int IDMEFPath::GetDepth()
+unsigned int IDMEFPath::getDepth() const
 {
         return idmef_path_get_depth(_path);
 }
@@ -381,3 +376,10 @@ IDMEFPath &IDMEFPath::operator=(const IDMEFPath &path)
 
         return *this;
 }
+
+
+IDMEFPath::operator idmef_path_t *() const
+{
+        return _path;
+}
+
