@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2008-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2008-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -57,11 +57,11 @@ IDMEFTime::IDMEFTime(idmef_time_t *time)
 }
 
 
-IDMEFTime::IDMEFTime(const time_t *time)
+IDMEFTime::IDMEFTime(time_t time)
 {
         int ret;
 
-        ret = idmef_time_new_from_time(&_time, time);
+        ret = idmef_time_new_from_time(&_time, &time);
         if ( ret < 0 )
                 throw PreludeError(ret);
 }
@@ -88,13 +88,13 @@ IDMEFTime::IDMEFTime(const struct timeval *tv)
 
 
 
-void IDMEFTime::Set(const time_t *t)
+void IDMEFTime::set(const time_t *t)
 {
         idmef_time_set_from_time(_time, t);
 }
 
 
-void IDMEFTime::Set(const struct timeval *tv)
+void IDMEFTime::set(const struct timeval *tv)
 {
         int ret;
 
@@ -105,7 +105,7 @@ void IDMEFTime::Set(const struct timeval *tv)
 
 
 
-void IDMEFTime::Set(const char *str)
+void IDMEFTime::set(const char *str)
 {
         int ret;
 
@@ -115,7 +115,7 @@ void IDMEFTime::Set(const char *str)
 }
 
 
-void IDMEFTime::Set()
+void IDMEFTime::set()
 {
         int ret;
 
@@ -125,43 +125,43 @@ void IDMEFTime::Set()
 }
 
 
-void IDMEFTime::SetSec(uint32_t sec)
+void IDMEFTime::setSec(uint32_t sec)
 {
         idmef_time_set_sec(_time, sec);
 }
 
 
-void IDMEFTime::SetUSec(uint32_t usec)
+void IDMEFTime::setUSec(uint32_t usec)
 {
         idmef_time_set_usec(_time, usec);
 }
 
 
-void IDMEFTime::SetGmtOffset(int32_t gmtoff)
+void IDMEFTime::setGmtOffset(int32_t gmtoff)
 {
         idmef_time_set_gmt_offset(_time, gmtoff);
 }
 
 
-uint32_t IDMEFTime::GetSec() const
+uint32_t IDMEFTime::getSec() const
 {
         return idmef_time_get_sec(_time);
 }
 
 
-uint32_t IDMEFTime::GetUSec() const
+uint32_t IDMEFTime::getUSec() const
 {
         return idmef_time_get_usec(_time);
 }
 
 
-int32_t IDMEFTime::GetGmtOffset() const
+int32_t IDMEFTime::getGmtOffset() const
 {
         return idmef_time_get_gmt_offset(_time);
 }
 
 
-IDMEFTime IDMEFTime::Clone()
+IDMEFTime IDMEFTime::clone() const
 {
         int ret;
         idmef_time_t *clone;
@@ -174,7 +174,7 @@ IDMEFTime IDMEFTime::Clone()
 }
 
 
-const std::string IDMEFTime::ToString() const
+const std::string IDMEFTime::toString() const
 {
         int ret;
         std::string cs;
@@ -195,37 +195,37 @@ const std::string IDMEFTime::ToString() const
 }
 
 
-bool IDMEFTime::operator <= (const IDMEFTime &time)
+bool IDMEFTime::operator <= (const IDMEFTime &time) const
 {
         return ( (double) *this <= (double) time );
 }
 
 
-bool IDMEFTime::operator < (const IDMEFTime &time)
+bool IDMEFTime::operator < (const IDMEFTime &time) const
 {
         return ( (double) *this < (double) time );
 }
 
 
-bool IDMEFTime::operator >= (const IDMEFTime &time)
+bool IDMEFTime::operator >= (const IDMEFTime &time) const
 {
         return ( (double) *this >= (double) time );
 }
 
 
-bool IDMEFTime::operator > (const IDMEFTime &time)
+bool IDMEFTime::operator > (const IDMEFTime &time) const
 {
         return ( (double) *this > (double) time );
 }
 
 
-bool IDMEFTime::operator != (const IDMEFTime &time)
+bool IDMEFTime::operator != (const IDMEFTime &time) const
 {
         return ( (double) *this != (double) time );
 }
 
 
-bool IDMEFTime::operator == (const IDMEFTime &time)
+bool IDMEFTime::operator == (const IDMEFTime &time) const
 {
         return ( (double) *this == (double) time );
 }
@@ -233,25 +233,25 @@ bool IDMEFTime::operator == (const IDMEFTime &time)
 
 IDMEFTime::operator int() const
 {
-        return GetSec();
+        return getSec();
 }
 
 
 IDMEFTime::operator long() const
 {
-        return GetSec();
+        return getSec();
 }
 
 
 IDMEFTime::operator double() const
 {
-        return GetSec() + (GetUSec() * 1e-6);
+        return getSec() + (getUSec() * 1e-6);
 }
 
 
 IDMEFTime::operator const std::string() const
 {
-        return ToString();
+        return toString();
 }
 
 
