@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2008-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2008-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,17 +15,19 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
+#include <string.h>
 #include <sstream>
 
 #include "prelude-error.hxx"
 #include "idmef.hxx"
 #include "idmef-value.hxx"
+#include "idmef-criteria.hxx"
 
 using namespace Prelude;
 
@@ -37,13 +39,13 @@ IDMEFValue::~IDMEFValue()
 }
 
 
-idmef_value_type_id_t IDMEFValue::GetType() const
+IDMEFValue::IDMEFValueTypeEnum IDMEFValue::getType() const
 {
-        return idmef_value_get_type(_value);
+        return (IDMEFValue::IDMEFValueTypeEnum) idmef_value_get_type(_value);
 }
 
 
-bool IDMEFValue::IsNull() const
+bool IDMEFValue::isNull() const
 {
         return (! _value) ? TRUE : FALSE;
 }
@@ -66,12 +68,12 @@ IDMEFValue::IDMEFValue(const IDMEFValue &value)
 }
 
 
-void IDMEFValue::_InitFromString(const char *value)
+void IDMEFValue::_InitFromString(const char *value, size_t len)
 {
         int ret;
         prelude_string_t *str;
 
-        ret = prelude_string_new_dup(&str, value);
+        ret = prelude_string_new_dup_fast(&str, value, len);
         if ( ret < 0 )
                 throw PreludeError(ret);
 
@@ -85,45 +87,13 @@ void IDMEFValue::_InitFromString(const char *value)
 
 IDMEFValue::IDMEFValue(const char *value)
 {
-        _InitFromString(value);
+        _InitFromString(value, strlen(value));
 }
 
 
-IDMEFValue::IDMEFValue(std::string value)
+IDMEFValue::IDMEFValue(const std::string &value)
 {
-        _InitFromString(value.c_str());
-}
-
-
-IDMEFValue::IDMEFValue(int8_t value)
-{
-        int ret = idmef_value_new_int8(&_value, value);
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-
-IDMEFValue::IDMEFValue(uint8_t value)
-{
-        int ret = idmef_value_new_uint8(&_value, value);
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-
-IDMEFValue::IDMEFValue(int16_t value)
-{
-        int ret = idmef_value_new_int16(&_value, value);
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-
-IDMEFValue::IDMEFValue(uint16_t value)
-{
-        int ret = idmef_value_new_uint16(&_value, value);
-        if ( ret < 0 )
-                throw PreludeError(ret);
+        _InitFromString(value.c_str(), value.size());
 }
 
 
@@ -135,14 +105,6 @@ IDMEFValue::IDMEFValue(int32_t value)
 }
 
 
-IDMEFValue::IDMEFValue(uint32_t value)
-{
-        int ret = idmef_value_new_uint32(&_value, value);
-        if ( ret < 0 )
-                throw PreludeError(ret);
-}
-
-
 IDMEFValue::IDMEFValue(int64_t value)
 {
         int ret = idmef_value_new_int64(&_value, value);
@@ -183,7 +145,42 @@ IDMEFValue::IDMEFValue(IDMEFTime &time)
 }
 
 
-IDMEFValue::IDMEFValue(std::vector<IDMEFValue> value)
+IDMEFValue::IDMEFValue(IDMEF *idmef)
+{
+        int ret;
+        idmef_value_t *v = NULL;
+
+        if (  idmef ) {
+                ret = idmef_value_new_class(&v, idmef->getId(), idmef_object_ref((idmef_object_t *) *idmef));
+                if ( ret < 0 )
+                       throw PreludeError(ret);
+        }
+
+        _value = v;
+}
+
+
+IDMEFValue::IDMEFValue(const std::vector<IDMEF> &value)
+{
+        int ret;
+        idmef_value_t *vitem;
+        std::vector<Prelude::IDMEF>::const_iterator i;
+
+        ret = idmef_value_new_list(&_value);
+        if ( ret < 0 )
+                throw PreludeError(ret);
+
+        for ( i = value.begin(); i != value.end(); i++ ) {
+                ret = idmef_value_new_class(&vitem, i->getId(), idmef_object_ref((idmef_object_t *) *i));
+                if ( ret < 0 )
+                        throw PreludeError(ret);
+
+                idmef_value_list_add(_value, vitem);
+        }
+}
+
+
+IDMEFValue::IDMEFValue(const std::vector<IDMEFValue> &value)
 {
         int ret;
         std::vector<Prelude::IDMEFValue>::const_iterator i;
@@ -202,13 +199,19 @@ IDMEFValue::IDMEFValue(idmef_value_t *value)
 }
 
 
-int IDMEFValue::Match(const IDMEFValue &value, int op)
+int IDMEFValue::match(const IDMEFValue &value, int op) const
 {
-        return idmef_value_match(this->_value, value._value, (idmef_criterion_operator_t) op);
+        int ret;
+
+        ret = idmef_value_match(this->_value, value._value, (idmef_criterion_operator_t) op);
+        if ( ret < 0 )
+                throw PreludeError(ret);
+
+        return ret;
 }
 
 
-IDMEFValue IDMEFValue::Clone() const
+IDMEFValue IDMEFValue::clone() const
 {
         int ret;
         idmef_value_t *clone;
@@ -221,11 +224,37 @@ IDMEFValue IDMEFValue::Clone() const
 }
 
 
+const std::string IDMEFValue::toString() const
+{
+        int ret;
+        std::string s;
+        prelude_string_t *str;
+
+        ret = prelude_string_new(&str);
+        if ( ret < 0 )
+                throw PreludeError(ret);
+
+        ret = idmef_value_to_string(_value, str);
+        if ( ret < 0 ) {
+                prelude_string_destroy(str);
+                throw PreludeError(ret);
+        }
+
+        s = prelude_string_get_string(str);
+        prelude_string_destroy(str);
+
+        return s;
+}
+
+
 static int iterate_cb(idmef_value_t *value, void *extra)
 {
         std::vector<IDMEFValue> *vlist = (std::vector<IDMEFValue> *) extra;
 
-        vlist->push_back(IDMEFValue(idmef_value_ref(value)));
+        if ( value )
+                value = idmef_value_ref(value);
+
+        vlist->push_back(IDMEFValue(value));
 
         return 0;
 }
@@ -238,9 +267,9 @@ IDMEFValue::operator std::vector<IDMEFValue> () const
         if ( ! _value )
                 return vlist;
 
-        if ( GetType() != IDMEF_VALUE_TYPE_LIST ) {
+        if ( getType() != TYPE_LIST ) {
                 std::stringstream s;
-                s << "Left value doesn't fit '" << idmef_value_type_to_string(GetType()) << "' requirement";
+                s << "Left value doesn't fit '" << idmef_value_type_to_string((idmef_value_type_id_t) getType()) << "' requirement";
                 throw PreludeError(s.str());
         }
 
@@ -254,9 +283,9 @@ IDMEFValue::operator IDMEFTime () const
 {
         prelude_except_if_fail(_value);
 
-        if ( GetType() != IDMEF_VALUE_TYPE_TIME ) {
+        if ( getType() != TYPE_TIME ) {
                 std::stringstream s;
-                s << "Left value doesn't fit '" << idmef_value_type_to_string(GetType()) << "' requirement";
+                s << "Left value doesn't fit '" << idmef_value_type_to_string((idmef_value_type_id_t) getType()) << "' requirement";
                 throw PreludeError(s.str());
         }
 
@@ -267,43 +296,40 @@ IDMEFValue::operator IDMEFTime () const
 
 IDMEFValue::operator int32_t () const
 {
-        idmef_value_type_id_t vtype;
+        IDMEFValueTypeEnum vtype;
 
         prelude_except_if_fail(_value);
-        vtype = GetType();
+        vtype = getType();
 
-        if ( vtype == IDMEF_VALUE_TYPE_INT8 )
+        if ( vtype == TYPE_INT8 )
                 return idmef_value_get_int8(_value);
 
-        else if ( vtype == IDMEF_VALUE_TYPE_UINT8 )
+        else if ( vtype == TYPE_UINT8 )
                 return idmef_value_get_uint8(_value);
 
-        else if ( vtype == IDMEF_VALUE_TYPE_INT16 )
+        else if ( vtype == TYPE_INT16 )
                 return idmef_value_get_int16(_value);
 
-        else if ( vtype == IDMEF_VALUE_TYPE_UINT16 )
+        else if ( vtype == TYPE_UINT16 )
                 return idmef_value_get_uint16(_value);
 
-        else if ( vtype == IDMEF_VALUE_TYPE_INT32 )
+        else if ( vtype == TYPE_INT32 )
                 return idmef_value_get_int32(_value);
 
-        else if ( vtype == IDMEF_VALUE_TYPE_ENUM )
+        else if ( vtype == TYPE_ENUM )
                 return idmef_value_get_enum(_value);
 
         std::stringstream s;
-        s << "Left value doesn't fit '" << idmef_value_type_to_string(vtype) << "' requirement";
+        s << "Left value doesn't fit '" << idmef_value_type_to_string((idmef_value_type_id_t) vtype) << "' requirement";
         throw PreludeError(s.str());
 }
 
 
 IDMEFValue::operator uint32_t () const
 {
-        idmef_value_type_id_t vtype;
-
         prelude_except_if_fail(_value);
-        vtype = GetType();
 
-        if ( vtype == IDMEF_VALUE_TYPE_UINT32 )
+        if ( getType() == TYPE_UINT32 )
                 return idmef_value_get_uint32(_value);
         else
                 return (int32_t) *this;
@@ -312,12 +338,9 @@ IDMEFValue::operator uint32_t () const
 
 IDMEFValue::operator int64_t () const
 {
-        idmef_value_type_id_t vtype;
-
         prelude_except_if_fail(_value);
-        vtype = GetType();
 
-        if ( vtype == IDMEF_VALUE_TYPE_INT64 )
+        if ( getType() == TYPE_INT64 )
                 return idmef_value_get_int64(_value);
         else
                 return (uint32_t) *this;
@@ -326,12 +349,9 @@ IDMEFValue::operator int64_t () const
 
 IDMEFValue::operator uint64_t () const
 {
-        idmef_value_type_id_t vtype;
-
         prelude_except_if_fail(_value);
-        vtype = GetType();
 
-        if ( vtype == IDMEF_VALUE_TYPE_UINT64 )
+        if ( getType() == TYPE_UINT64 )
                 return idmef_value_get_uint64(_value);
         else
                 return (int64_t) *this;
@@ -340,15 +360,15 @@ IDMEFValue::operator uint64_t () const
 
 IDMEFValue::operator float () const
 {
-        idmef_value_type_id_t vtype;
+        IDMEFValueTypeEnum vtype;
 
         prelude_except_if_fail(_value);
-        vtype = GetType();
+        vtype = getType();
 
-        if ( vtype == IDMEF_VALUE_TYPE_FLOAT )
+        if ( vtype == TYPE_FLOAT )
                 return idmef_value_get_float(_value);
 
-        else if ( vtype == IDMEF_VALUE_TYPE_DATA ) {
+        else if ( vtype == TYPE_DATA ) {
                 idmef_data_t *d = idmef_value_get_data(_value);
 
                 if ( idmef_data_get_type(d) == IDMEF_DATA_TYPE_FLOAT )
@@ -356,21 +376,17 @@ IDMEFValue::operator float () const
         }
 
         std::stringstream s;
-        s << "Left value doesn't fit '" << idmef_value_type_to_string(vtype) << "' requirement";
+        s << "Left value doesn't fit '" << idmef_value_type_to_string((idmef_value_type_id_t) vtype) << "' requirement";
         throw PreludeError(s.str());
 }
 
 
 IDMEFValue::operator double () const
 {
-        idmef_value_type_id_t vtype;
-
         prelude_except_if_fail(_value);
-        vtype = GetType();
 
-        if ( vtype == IDMEF_VALUE_TYPE_DOUBLE )
+        if ( getType() == TYPE_DOUBLE )
                 return idmef_value_get_double(_value);
-
         else
                 return (float) *this;
 }
@@ -381,16 +397,16 @@ std::string IDMEFValue::convert_string() const
         std::stringstream s;
         prelude_except_if_fail(_value);
 
-        if ( GetType() == IDMEF_VALUE_TYPE_STRING )
+        if ( getType() == TYPE_STRING )
                 return prelude_string_get_string(idmef_value_get_string(_value));
 
-        else if ( GetType() == IDMEF_VALUE_TYPE_TIME )
+        else if ( getType() == TYPE_TIME )
                 return IDMEFTime(idmef_time_ref(idmef_value_get_time(_value)));
 
-        else if ( GetType() == IDMEF_VALUE_TYPE_ENUM )
+        else if ( getType() == TYPE_ENUM )
                 return idmef_class_enum_to_string(idmef_value_get_class(_value), idmef_value_get_enum(_value));
 
-        else if ( GetType() == IDMEF_VALUE_TYPE_DATA ) {
+        else if ( getType() == TYPE_DATA ) {
                 idmef_data_t *d = idmef_value_get_data(_value);
                 idmef_data_type_t t = idmef_data_get_type(d);
 
@@ -407,13 +423,8 @@ std::string IDMEFValue::convert_string() const
                         return s.str();
                 }
 
-                else if ( t == IDMEF_DATA_TYPE_UINT32 ) {
-                        s << idmef_data_get_uint32(d);
-                        return s.str();
-                }
-
-                else if ( t == IDMEF_DATA_TYPE_UINT64 ) {
-                        s << idmef_data_get_uint64(d);
+                else if ( t == IDMEF_DATA_TYPE_INT ) {
+                        s << idmef_data_get_int(d);
                         return s.str();
                 }
 
@@ -423,7 +434,7 @@ std::string IDMEFValue::convert_string() const
                 }
         }
 
-        s << "Left value doesn't fit '" << idmef_value_type_to_string(GetType()) << "' requirement";
+        s << "Left value doesn't fit '" << idmef_value_type_to_string((idmef_value_type_id_t) getType()) << "' requirement";
         throw PreludeError(s.str());
 }
 
@@ -453,3 +464,45 @@ IDMEFValue::operator idmef_value_t *() const
 {
         return _value;
 }
+
+
+bool IDMEFValue::operator <= (const IDMEFValue &value) const
+{
+        return this->match(value, IDMEFCriterion::OPERATOR_LESSER|IDMEFCriterion::OPERATOR_EQUAL);
+}
+
+
+bool IDMEFValue::operator >= (const IDMEFValue &value) const
+{
+        return this->match(value, IDMEFCriterion::OPERATOR_GREATER|IDMEFCriterion::OPERATOR_EQUAL);
+}
+
+
+bool IDMEFValue::operator < (const IDMEFValue &value) const
+{
+        return this->match(value, IDMEFCriterion::OPERATOR_LESSER);
+}
+
+
+bool IDMEFValue::operator > (const IDMEFValue &value) const
+{
+        return this->match(value, IDMEFCriterion::OPERATOR_GREATER);
+}
+
+
+bool IDMEFValue::operator == (const IDMEFValue &value) const
+{
+        return this->match(value, IDMEFCriterion::OPERATOR_EQUAL);
+}
+
+
+bool IDMEFValue::operator == (const std::vector<IDMEFValue> &vlist) const
+{
+        return this->match(vlist, IDMEFCriterion::OPERATOR_EQUAL);
+}
+
+
+bool IDMEFValue::operator != (const IDMEFValue &value) const
+{
+        return this->match(value, IDMEFCriterion::OPERATOR_NOT|IDMEFCriterion::OPERATOR_EQUAL);
+}
