@@ -1,3 +1,26 @@
+/*****
+*
+* Copyright (C) 2009-2016 CS-SI. All Rights Reserved.
+* Author: Yoann Vandoorselaere <yoannv@gmail.com>
+*
+* This file is part of the Prelude library.
+*
+* This program is free software; you can redistribute it and/or modify
+* it under the terms of the GNU General Public License as published by
+* the Free Software Foundation; either version 2, or (at your option)
+* any later version.
+*
+* This program is distributed in the hope that it will be useful,
+* but WITHOUT ANY WARRANTY; without even the implied warranty of
+* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+* GNU General Public License for more details.
+*
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+*
+*****/
+
 #include "idmef.hxx"
 #include "prelude-error.hxx"
 #include "prelude-client.hxx"
@@ -34,11 +57,11 @@ Client::~Client()
 }
 
 
-void Client::Start()
+void Client::start()
 {
         int ret;
 
-        Init();
+        init();
 
         ret = prelude_client_start(_client);
         if ( ret < 0 )
@@ -46,7 +69,7 @@ void Client::Start()
 }
 
 
-void Client::Init()
+void Client::init()
 {
         int ret;
 
@@ -58,19 +81,19 @@ void Client::Init()
 }
 
 
-prelude_client_t *Client::GetClient()
+prelude_client_t *Client::getClient() const
 {
         return _client;
 }
 
 
-void Client::SendIDMEF(const IDMEF &message)
+void Client::sendIDMEF(const IDMEF &message)
 {
-        prelude_client_send_idmef(_client, message);
+        prelude_client_send_idmef(_client, (idmef_message_t *) (idmef_object_t *) message);
 }
 
 
-int Client::RecvIDMEF(Prelude::IDMEF &idmef, int timeout)
+int Client::recvIDMEF(Prelude::IDMEF &idmef, int timeout)
 {
         int ret;
         idmef_message_t *idmef_p;
@@ -82,19 +105,19 @@ int Client::RecvIDMEF(Prelude::IDMEF &idmef, int timeout)
         else if ( ret == 0 )
                 return 0;
 
-        idmef = IDMEF(idmef_p);
+        idmef = IDMEF((idmef_object_t *)idmef_p);
 
         return 1;
 }
 
 
-int Client::GetFlags()
+int Client::getFlags() const
 {
         return prelude_client_get_flags(_client);
 }
 
 
-void Client::SetFlags(int flags)
+void Client::setFlags(int flags)
 {
         int ret;
 
@@ -104,25 +127,25 @@ void Client::SetFlags(int flags)
 }
 
 
-int Client::GetRequiredPermission()
+int Client::getRequiredPermission() const
 {
         return prelude_client_get_required_permission(_client);
 }
 
 
-void Client::SetRequiredPermission(int permission)
+void Client::setRequiredPermission(int permission)
 {
         prelude_client_set_required_permission(_client, (prelude_connection_permission_t) permission);
 }
 
 
-const char *Client::GetConfigFilename()
+const char *Client::getConfigFilename() const
 {
         return prelude_client_get_config_filename(_client);
 }
 
 
-void Client::SetConfigFilename(const char *name)
+void Client::setConfigFilename(const char *name)
 {
         int ret;
 
@@ -132,13 +155,13 @@ void Client::SetConfigFilename(const char *name)
 }
 
 
-ConnectionPool &Client::GetConnectionPool()
+ConnectionPool &Client::getConnectionPool()
 {
         return _pool;
 }
 
 
-void Client::SetConnectionPool(ConnectionPool pool)
+void Client::setConnectionPool(ConnectionPool pool)
 {
         _pool = pool;
         prelude_client_set_connection_pool(_client, prelude_connection_pool_ref(pool));
@@ -147,7 +170,7 @@ void Client::SetConnectionPool(ConnectionPool pool)
 
 Client &Client::operator << (IDMEF &idmef)
 {
-        SendIDMEF(idmef);
+        sendIDMEF(idmef);
         return *this;
 }
 
@@ -156,14 +179,14 @@ Client &Client::operator >> (IDMEF &idmef)
 {
         int ret;
 
-        ret = RecvIDMEF(idmef, _recv_timeout);
+        ret = recvIDMEF(idmef, _recv_timeout);
         if ( ret <= 0 )
                 throw PreludeError(ret);
 
         return *this;
 }
 
-Client &Client::SetRecvTimeout(Client &c, int timeout)
+Client &Client::setRecvTimeout(Client &c, int timeout)
 {
         c._recv_timeout = timeout;
         return c;
