@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2008-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2008-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -68,19 +68,19 @@ Connection::Connection(prelude_connection_t *con, bool own_data)
 }
 
 
-prelude_connection_t *Connection::GetConnection()
+prelude_connection_t *Connection::getConnection() const
 {
         return _con;
 }
 
 
-void Connection::Close()
+void Connection::close()
 {
         prelude_connection_close(_con);
 }
 
 
-void Connection::Connect(ClientProfile &cp, int permission)
+void Connection::connect(ClientProfile &cp, int permission)
 {
         int ret;
 
@@ -90,79 +90,79 @@ void Connection::Connect(ClientProfile &cp, int permission)
 }
 
 
-void Connection::SetState(int state)
+void Connection::setState(int state)
 {
         prelude_connection_set_state(_con, (prelude_connection_state_t) state);
 }
 
 
-int Connection::GetState()
+int Connection::getState() const
 {
         return prelude_connection_get_state(_con);
 }
 
 
-void Connection::SetData(void *data)
+void Connection::setData(void *data)
 {
         prelude_connection_set_data(_con, data);
 }
 
 
-void *Connection::GetData()
+void *Connection::getData() const
 {
         return prelude_connection_get_data(_con);
 }
 
 
-int Connection::GetPermission()
+int Connection::getPermission() const
 {
         return prelude_connection_get_permission(_con);
 }
 
 
-void Connection::SetPeerAnalyzerid(uint64_t analyzerid)
+void Connection::setPeerAnalyzerid(uint64_t analyzerid)
 {
         prelude_connection_set_peer_analyzerid(_con, analyzerid);
 }
 
 
-uint64_t Connection::GetPeerAnalyzerid()
+uint64_t Connection::getPeerAnalyzerid() const
 {
         return prelude_connection_get_peer_analyzerid(_con);
 }
 
 
-const char *Connection::GetLocalAddr()
+const char *Connection::getLocalAddr() const
 {
         return prelude_connection_get_local_addr(_con);
 }
 
 
-unsigned int Connection::GetLocalPort()
+unsigned int Connection::getLocalPort() const
 {
         return prelude_connection_get_local_port(_con);
 }
 
 
-const char *Connection::GetPeerAddr()
+const char *Connection::getPeerAddr() const
 {
         return prelude_connection_get_peer_addr(_con);
 }
 
 
-unsigned int Connection::GetPeerPort()
+unsigned int Connection::getPeerPort() const
 {
         return prelude_connection_get_peer_port(_con);
 }
 
 
-int Connection::GetFd()
+int Connection::getFd() const
 {
         return prelude_io_get_fd(prelude_connection_get_fd(_con));
 }
 
 
-Prelude::IDMEF Connection::RecvIDMEF()
+Prelude::IDMEF Connection::recvIDMEF()
 {
         int ret;
         idmef_message_t *idmef;
@@ -171,11 +171,11 @@ Prelude::IDMEF Connection::RecvIDMEF()
         if ( ret < 0 )
                 throw PreludeError(ret);
 
-        return IDMEF(idmef);
+        return IDMEF((idmef_object_t *) idmef);
 }
 
 
-bool Connection::IsAlive()
+bool Connection::isAlive() const
 {
         return prelude_connection_is_alive(_con);
 }
