@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2008-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2008-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -31,7 +31,13 @@
 using namespace Prelude;
 
 
-PreludeError::PreludeError(const std::string message) throw()
+PreludeError::PreludeError(void) throw()
+{
+        _error = -1;
+}
+
+
+PreludeError::PreludeError(const std::string &message) throw()
 {
         _error = -1;
         _message = message;
@@ -51,13 +57,19 @@ const char *PreludeError::what() const throw()
 }
 
 
-PreludeError::operator const char *()
+int PreludeError::getCode() const
+{
+        return prelude_error_get_code(_error);
+}
+
+
+PreludeError::operator const char *() const
 {
         return _message.c_str();
 }
 
 
-PreludeError::operator const std::string() const
+PreludeError::operator int () const
 {
-        return _message;
+        return prelude_error_get_code(_error);
 }
