@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -39,7 +39,10 @@
 
 #include <gcrypt.h>
 #include <gnutls/gnutls.h>
-#include <gnutls/extra.h>
+
+#ifdef NEED_GNUTLS_EXTRA
+# include <gnutls/extra.h>
+#endif
 
 #include "prelude-client.h"
 #include "prelude-error.h"
@@ -53,11 +56,11 @@
 
 
 static const char *one_shot_passwd;
-static gnutls_anon_server_credentials anoncred;
+static gnutls_anon_server_credentials_t anoncred;
 
 
 #ifdef GNUTLS_SRP_ENABLED
- static gnutls_srp_server_credentials srpcred;
+ static gnutls_srp_server_credentials_t srpcred;
 #endif
 
 
@@ -97,10 +100,10 @@ static int anon_check_passwd(prelude_io_t *fd)
 }
 
 
-static inline gnutls_transport_ptr fd_to_ptr(int fd)
+static inline gnutls_transport_ptr_t fd_to_ptr(int fd)
 {
         union {
-                gnutls_transport_ptr ptr;
+                gnutls_transport_ptr_t ptr;
                 int fd;
         } data;
 
@@ -110,10 +113,10 @@ static inline gnutls_transport_ptr fd_to_ptr(int fd)
 }
 
 
-static inline int ptr_to_fd(gnutls_transport_ptr ptr)
+static inline int ptr_to_fd(gnutls_transport_ptr_t ptr)
 {
         union {
-                gnutls_transport_ptr ptr;
+                gnutls_transport_ptr_t ptr;
                 int fd;
         } data;
 
@@ -123,34 +126,45 @@ static inline int ptr_to_fd(gnutls_transport_ptr ptr)
 }
 
 
-static ssize_t tls_pull(gnutls_transport_ptr fd, void *buf, size_t count)
+static ssize_t tls_pull(gnutls_transport_ptr_t fd, void *buf, size_t count)
 {
         return read(ptr_to_fd(fd), buf, count);
 }
 
 
 
-static ssize_t tls_push(gnutls_transport_ptr fd, const void *buf, size_t count)
+static ssize_t tls_push(gnutls_transport_ptr_t fd, const void *buf, size_t count)
 {
         return write(ptr_to_fd(fd), buf, count);
 }
 
 
-static gnutls_session new_tls_session(int sock)
+static gnutls_session_t new_tls_session(int sock)
 {
         int ret;
-        gnutls_session session;
-        const int kx_priority[] = {
-                GNUTLS_KX_ANON_DH,
+        gnutls_session_t session;
+        const char *err;
+
+#if defined LIBGNUTLS_VERSION_MAJOR && LIBGNUTLS_VERSION_MAJOR >= 3
+# define TLS_DH_STR "+ANON-ECDH:+ANON-DH"
+#else
+# define TLS_DH_STR "+ANON-DH"
+#endif
+
 #ifdef GNUTLS_SRP_ENABLED
-                GNUTLS_KX_SRP, GNUTLS_KX_SRP_DSS, GNUTLS_KX_SRP_RSA,
+        const char *pstring = "NORMAL:+SRP:+SRP-DSS:+SRP-RSA:" TLS_DH_STR;
+#else
+        const char *pstring = "NORMAL:" TLS_DH_STR;
 #endif
-                0 };
 
         gnutls_init(&session, GNUTLS_SERVER);
-
         gnutls_set_default_priority(session);
-        gnutls_kx_set_priority(session, kx_priority);
+
+        ret = gnutls_priority_set_direct(session, pstring, &err);
+        if (ret < 0) {
+                fprintf(stderr, "TLS priority syntax error at: %s\n", err);
+                return NULL;
+        }
 
 #ifdef GNUTLS_SRP_ENABLED
         gnutls_credentials_set(session, GNUTLS_CRD_SRP, srpcred);
@@ -175,9 +189,9 @@ static gnutls_session new_tls_session(int sock)
 
 
 static int handle_client_connection(const char *srcinfo, prelude_client_profile_t *cp, prelude_io_t *fd,
-                                    gnutls_x509_privkey key, gnutls_x509_crt cacrt, gnutls_x509_crt crt)
+                                    gnutls_x509_privkey_t key, gnutls_x509_crt_t cacrt, gnutls_x509_crt_t crt)
 {
-        gnutls_session session;
+        gnutls_session_t session;
 
         session = new_tls_session(prelude_io_get_fd(fd));
         if ( ! session )
@@ -194,7 +208,7 @@ static int handle_client_connection(const char *srcinfo, prelude_client_profile_
 
 
 static int process_event(prelude_client_profile_t *cp, int server_sock, prelude_io_t *fd,
-                         gnutls_x509_privkey key, gnutls_x509_crt cacrt, gnutls_x509_crt crt)
+                         gnutls_x509_privkey_t key, gnutls_x509_crt_t cacrt, gnutls_x509_crt_t crt)
 {
         char buf[512];
         void *inaddr;
@@ -242,7 +256,7 @@ static int process_event(prelude_client_profile_t *cp, int server_sock, prelude_
 
 static int wait_connection(prelude_client_profile_t *cp, int sock,
                            struct pollfd *pfd, size_t size, int keepalive,
-                           gnutls_x509_privkey key, gnutls_x509_crt cacrt, gnutls_x509_crt crt)
+                           gnutls_x509_privkey_t key, gnutls_x509_crt_t cacrt, gnutls_x509_crt_t crt)
 {
         size_t i;
         prelude_io_t *fd;
@@ -284,7 +298,7 @@ static int setup_server(const char *addr, unsigned int port, struct pollfd *pfd,
         size_t i = 0;
         char buf[1024];
         struct addrinfo hints, *ai, *ai_start;
-        int sock, ret, on = 1, prev_family = PF_UNSPEC;
+        int sock, ret, on = 1;
 
         snprintf(buf, sizeof(buf), "%u", port);
         memset(&hints, 0, sizeof(hints));
@@ -292,7 +306,7 @@ static int setup_server(const char *addr, unsigned int port, struct pollfd *pfd,
         hints.ai_flags = AI_PASSIVE;
         hints.ai_socktype = SOCK_STREAM;
         hints.ai_protocol = IPPROTO_TCP;
-        hints.ai_family = PF_UNSPEC;
+        hints.ai_family = AF_UNSPEC;
 
 #ifdef AI_ADDRCONFIG
         hints.ai_flags |= AI_ADDRCONFIG;
@@ -308,15 +322,35 @@ static int setup_server(const char *addr, unsigned int port, struct pollfd *pfd,
         for ( ai_start = ai; ai && i < *size; ai = ai->ai_next ) {
                 inet_ntop(ai->ai_family, prelude_sockaddr_get_inaddr(ai->ai_addr), buf, sizeof(buf));
 
-                fprintf(stderr, "Waiting for peers install request on %s:%u...\n",
-                        buf, ntohs(((struct sockaddr_in *) ai->ai_addr)->sin_port));
-
                 sock = socket(ai->ai_family, ai->ai_socktype, ai->ai_protocol);
                 if ( sock < 0 ) {
                         fprintf(stderr, "could not open socket for '%s': %s.\n", buf, strerror(errno));
                         break;
                 }
 
+#ifdef IPV6_V6ONLY
+                /*
+                 * There is a problem on Linux system where getaddrinfo() return address in
+                 * the wrong sort order (IPv4 first, IPv6 next).
+                 *
+                 * As a result we first bind IPv4 addresses, but then we get an error for
+                 * dual-stacked addresses, when the IPv6 addresses come second. When an
+                 * address is dual-stacked, we thus end-up listening only to the IPv4
+                 * instance.
+                 *
+                 * The error happen on dual-stack Linux system, because mapping the IPv6
+                 * address will actually attempt to bind both the IPv4 and IPv6 address.
+                 *
+                 * In order to prevent this problem, we set the IPV6_V6ONLY option so that
+                 * only the IPv6 address will be bound.
+                 */
+                if ( ai->ai_family == AF_INET6 ) {
+                        ret = setsockopt(sock, IPPROTO_IPV6, IPV6_V6ONLY, (void *) &on, sizeof(int));
+                        if ( ret < 0 )
+                                fprintf(stderr, "could not set IPV6_V6ONLY: %s.\n", strerror(errno));
+                }
+#endif
+
                 ret = setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (void *) &on, sizeof(int));
                 if ( ret < 0 )
                         fprintf(stderr, "could not set SO_REUSEADDR: %s.\n", strerror(errno));
@@ -324,16 +358,6 @@ static int setup_server(const char *addr, unsigned int port, struct pollfd *pfd,
                 ret = bind(sock, ai->ai_addr, ai->ai_addrlen);
                 if ( ret < 0 ) {
                         close(sock);
-
-                        /*
-                         * More information on this at:
-                         * http://lists.debian.org/debian-ipv6/2001/01/msg00031.html
-                         */
-                        if ( errno == EADDRINUSE && ! addr && prev_family != PF_UNSPEC && ai->ai_family != prev_family ) {
-                                ret = 0;
-                                continue;
-                        }
-
                         fprintf(stderr, "could not bind to '%s': %s.\n", buf, strerror(errno));
                         break;
                 }
@@ -345,9 +369,11 @@ static int setup_server(const char *addr, unsigned int port, struct pollfd *pfd,
                         break;
                 }
 
+                fprintf(stderr, "Waiting for peers install request on %s:%u...\n",
+                        buf, ntohs(((struct sockaddr_in *) ai->ai_addr)->sin_port));
+
                 pfd[i].fd = sock;
                 pfd[i].events = POLLIN;
-                prev_family = ai->ai_family;
 
                 i++;
         }
@@ -367,7 +393,7 @@ static int setup_server(const char *addr, unsigned int port, struct pollfd *pfd,
 
 #ifdef GNUTLS_SRP_ENABLED
 
-static int copy_datum(gnutls_datum *dst, const gnutls_datum *src)
+static int copy_datum(gnutls_datum_t *dst, const gnutls_datum_t *src)
 {
         dst->size = src->size;
 
@@ -384,8 +410,8 @@ static int copy_datum(gnutls_datum *dst, const gnutls_datum *src)
 
 
 
-static int srp_callback(gnutls_session session, const char *username, gnutls_datum *salt,
-                        gnutls_datum *verifier, gnutls_datum *generator, gnutls_datum *prime)
+static int srp_callback(gnutls_session_t session, const char *username, gnutls_datum_t *salt,
+                        gnutls_datum_t *verifier, gnutls_datum_t *generator, gnutls_datum_t *prime)
 {
         int ret;
 
@@ -417,12 +443,12 @@ static int srp_callback(gnutls_session session, const char *username, gnutls_dat
 
 
 int server_create(prelude_client_profile_t *cp, const char *addr, unsigned int port,
-                  prelude_bool_t keepalive, const char *pass, gnutls_x509_privkey key, gnutls_x509_crt cacrt, gnutls_x509_crt crt)
+                  prelude_bool_t keepalive, const char *pass, gnutls_x509_privkey_t key, gnutls_x509_crt_t cacrt, gnutls_x509_crt_t crt)
 {
         int sock;
         size_t size;
         struct pollfd pfd[128];
-        gnutls_dh_params dh_params;
+        gnutls_dh_params_t dh_params;
 
 #ifdef GNUTLS_SRP_ENABLED
         int ret;
