@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,13 +15,13 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
 int server_create(prelude_client_profile_t *cp, const char *addr, unsigned int port,
-                  prelude_bool_t keepalive, const char *pass, gnutls_x509_privkey key,
-                  gnutls_x509_crt cacrt, gnutls_x509_crt crt);
+                  prelude_bool_t keepalive, const char *pass, gnutls_x509_privkey_t key,
+                  gnutls_x509_crt_t cacrt, gnutls_x509_crt_t crt);
 
