@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -34,6 +34,9 @@
 
 #include <gnutls/gnutls.h>
 #include <gnutls/x509.h>
+#ifdef HAVE_GNUTLS_PRIVKEY_ABSTRACT
+# include <gnutls/abstract.h>
+#endif
 #include <gcrypt.h>
 
 #include "prelude-client.h"
@@ -52,7 +55,32 @@ extern int authority_certificate_lifetime;
 extern int generated_certificate_lifetime;
 
 
-static int cmp_certificate_dn(gnutls_x509_crt crt, uint64_t wanted_dn, uint64_t wanted_issuer_dn)
+
+#ifdef HAVE_GNUTLS_PRIVKEY_ABSTRACT
+static gnutls_privkey_t get_privkey_from_x509(gnutls_x509_privkey_t x509key)
+{
+        int ret;
+        gnutls_privkey_t key;
+
+        ret = gnutls_privkey_init(&key);
+        if ( ret < 0 ) {
+                fprintf(stderr, "error creating abstract key: %s.\n", gnutls_strerror(ret));
+                return NULL;
+        }
+
+        ret = gnutls_privkey_import_x509(key, x509key, GNUTLS_PRIVKEY_IMPORT_AUTO_RELEASE);
+        if ( ret < 0 ) {
+                fprintf(stderr, "error importing x509 abstract key: %s.\n", gnutls_strerror(ret));
+                gnutls_privkey_deinit(key);
+                return NULL;
+        }
+
+        return key;
+}
+#endif
+
+
+static int cmp_certificate_dn(gnutls_x509_crt_t crt, uint64_t wanted_dn, uint64_t wanted_issuer_dn)
 {
         int ret;
         char buf[128];
@@ -102,8 +130,8 @@ static int remove_old_certificate(const char *filename, uint64_t dn, uint64_t is
         unsigned char *data, *datap;
         size_t size;
         FILE *fd;
-        gnutls_datum datum;
-        gnutls_x509_crt crt;
+        gnutls_datum_t datum;
+        gnutls_x509_crt_t crt;
         prelude_string_t *out;
 
         ret = _prelude_load_file(filename, &data, &size);
@@ -181,8 +209,8 @@ static int remove_old(const char *filename, unsigned char *buf, size_t size)
 {
         int ret;
         char out[512];
-        gnutls_datum data;
-        gnutls_x509_crt crt;
+        gnutls_datum_t data;
+        gnutls_x509_crt_t crt;
         uint64_t dn, issuer_dn;
 
         data.size = size;
@@ -287,11 +315,11 @@ static int save_buf(const char *filename, prelude_uid_t uid, prelude_gid_t gid,
 
 
 
-static gnutls_x509_crt generate_certificate(prelude_client_profile_t *cp, gnutls_x509_privkey key, int expire)
+static gnutls_x509_crt_t generate_certificate(prelude_client_profile_t *cp, gnutls_x509_privkey_t key, int expire)
 {
         int ret;
         char buf[1024];
-        gnutls_x509_crt crt;
+        gnutls_x509_crt_t crt;
         uint64_t analyzerid;
         size_t size = sizeof(buf);
 
@@ -343,14 +371,14 @@ static gnutls_x509_crt generate_certificate(prelude_client_profile_t *cp, gnutls
 
 
 
-static gnutls_x509_crt generate_signed_certificate(prelude_client_profile_t *cp,
-                                                   uint64_t analyzerid,
-                                                   gnutls_x509_crt ca_crt,
-                                                   gnutls_x509_privkey ca_key,
-                                                   gnutls_x509_crq crq)
+static gnutls_x509_crt_t generate_signed_certificate(prelude_client_profile_t *cp,
+                                                     uint64_t analyzerid,
+                                                     gnutls_x509_crt_t ca_crt,
+                                                     gnutls_x509_privkey_t ca_key,
+                                                     gnutls_x509_crq_t crq)
 {
         int ret;
-        gnutls_x509_crt crt;
+        gnutls_x509_crt_t crt;
         unsigned char buf[65535];
         size_t size = sizeof(buf);
 
@@ -398,10 +426,10 @@ static gnutls_x509_crt generate_signed_certificate(prelude_client_profile_t *cp,
 
 
 
-static gnutls_x509_crt generate_ca_certificate(prelude_client_profile_t *cp, gnutls_x509_privkey key)
+static gnutls_x509_crt_t generate_ca_certificate(prelude_client_profile_t *cp, gnutls_x509_privkey_t key)
 {
         int ret;
-        gnutls_x509_crt crt;
+        gnutls_x509_crt_t crt;
         unsigned int usage = 0;
 
         crt = generate_certificate(cp, key, authority_certificate_lifetime);
@@ -441,10 +469,10 @@ static void entropy_progress_cb(void *cb_data, const char *what, int printchar,
 }
 
 
-static gnutls_x509_privkey generate_private_key(void)
+static gnutls_x509_privkey_t generate_private_key(void)
 {
         int ret;
-        gnutls_x509_privkey key;
+        gnutls_x509_privkey_t key;
 
         ret = gnutls_x509_privkey_init(&key);
         if ( ret < 0 ) {
@@ -474,12 +502,15 @@ static gnutls_x509_privkey generate_private_key(void)
 
 
 
-static gnutls_x509_crq generate_certificate_request(prelude_client_profile_t *cp,
-                                                    prelude_connection_permission_t permission,
-                                                    gnutls_x509_privkey key, unsigned char *buf, size_t *size)
+static gnutls_x509_crq_t generate_certificate_request(prelude_client_profile_t *cp,
+                                                      prelude_connection_permission_t permission,
+                                                      gnutls_x509_privkey_t x509key, unsigned char *buf, size_t *size)
 {
         int ret;
-        gnutls_x509_crq crq;
+        gnutls_x509_crq_t crq;
+#ifdef HAVE_GNUTLS_PRIVKEY_ABSTRACT
+        gnutls_privkey_t key;
+#endif
 
         ret = gnutls_x509_crq_init(&crq);
         if ( ret < 0 ) {
@@ -487,7 +518,7 @@ static gnutls_x509_crq generate_certificate_request(prelude_client_profile_t *cp
                 return NULL;
         }
 
-        ret = gnutls_x509_crq_set_key(crq, key);
+        ret = gnutls_x509_crq_set_key(crq, x509key);
         if ( ret < 0 ) {
                 fprintf(stderr, "error setting certificate request key: %s.\n", gnutls_strerror(ret));
                 gnutls_x509_crq_deinit(crq);
@@ -501,13 +532,6 @@ static gnutls_x509_crq generate_certificate_request(prelude_client_profile_t *cp
                         fprintf(stderr, "error setting common name: %s.\n", gnutls_strerror(ret));
                         return NULL;
                 }
-
-                ret = snprintf((char *) buf, *size, "%s", prelude_client_profile_get_name(cp));
-				ret = gnutls_x509_crq_set_dn_by_oid(crq, GNUTLS_OID_X520_SURNAME, 0, buf, ret);
-				if ( ret < 0 ) {
-						fprintf(stderr, "error setting surName: %s.\n", gnutls_strerror(ret));
-						return NULL;
-				}
         }
 
         gnutls_x509_crq_set_version(crq, 3);
@@ -519,7 +543,14 @@ static gnutls_x509_crq generate_certificate_request(prelude_client_profile_t *cp
                 return NULL;
         }
 
-        ret = gnutls_x509_crq_sign(crq, key);
+#ifdef HAVE_GNUTLS_PRIVKEY_ABSTRACT
+        key = get_privkey_from_x509(x509key);
+        if ( ! key )
+                return NULL;
+        ret = gnutls_x509_crq_privkey_sign(crq, key, GNUTLS_DIG_SHA256, 0);
+#else
+        ret = gnutls_x509_crq_sign(crq, x509key);
+#endif
         if ( ret < 0 ) {
                 fprintf(stderr, "error signing certificate request: %s.\n", gnutls_strerror(ret));
                 gnutls_x509_crq_deinit(crq);
@@ -538,12 +569,12 @@ static gnutls_x509_crq generate_certificate_request(prelude_client_profile_t *cp
 
 
 
-static gnutls_x509_privkey gen_crypto(prelude_client_profile_t *cp,
-                                      const char *filename, prelude_uid_t uid, prelude_gid_t gid)
+static gnutls_x509_privkey_t gen_crypto(prelude_client_profile_t *cp,
+                                        const char *filename, prelude_uid_t uid, prelude_gid_t gid)
 {
         int ret;
         char buf[65535];
-        gnutls_x509_privkey key;
+        gnutls_x509_privkey_t key;
         size_t size = sizeof(buf);
 
         key = generate_private_key();
@@ -570,13 +601,13 @@ static gnutls_x509_privkey gen_crypto(prelude_client_profile_t *cp,
 
 
 
-gnutls_x509_privkey tls_load_privkey(prelude_client_profile_t *cp)
+gnutls_x509_privkey_t tls_load_privkey(prelude_client_profile_t *cp)
 {
         int ret;
         size_t size;
-        gnutls_datum data;
+        gnutls_datum_t data;
         char filename[256];
-        gnutls_x509_privkey key;
+        gnutls_x509_privkey_t key;
 
         prelude_client_profile_get_tls_key_filename(cp, filename, sizeof(filename));
 
@@ -627,7 +658,7 @@ static char ask_req_confirmation(void)
 
 
 
-static int check_req(const char *srcinfo, prelude_io_t *fd, gnutls_x509_crq crq, uint64_t *analyzerid)
+static int check_req(const char *srcinfo, prelude_io_t *fd, gnutls_x509_crq_t crq, uint64_t *analyzerid)
 {
         int ret;
         size_t size;
@@ -683,17 +714,17 @@ static int check_req(const char *srcinfo, prelude_io_t *fd, gnutls_x509_crq crq,
 
 
 int tls_handle_certificate_request(const char *srcinfo, prelude_client_profile_t *cp, prelude_io_t *fd,
-                                   gnutls_x509_privkey cakey, gnutls_x509_crt cacrt,
-                                   gnutls_x509_crt crt)
+                                   gnutls_x509_privkey_t cakey, gnutls_x509_crt_t cacrt,
+                                   gnutls_x509_crt_t crt)
 {
         ssize_t ret;
         size_t size;
         char buf[65535];
-        gnutls_datum data;
-        gnutls_x509_crq crq;
+        gnutls_datum_t data;
+        gnutls_x509_crq_t crq;
         unsigned char *rbuf;
         uint64_t analyzerid;
-        gnutls_x509_crt gencrt;
+        gnutls_x509_crt_t gencrt;
 
         /*
          * Read the client CRQ and generate a certificate for it.
@@ -759,13 +790,13 @@ int tls_handle_certificate_request(const char *srcinfo, prelude_client_profile_t
 
 
 int tls_request_certificate(prelude_client_profile_t *cp, prelude_io_t *fd,
-                            gnutls_x509_privkey key, prelude_connection_permission_t permission)
+                            gnutls_x509_privkey_t key, prelude_connection_permission_t permission)
 {
         ssize_t ret;
         ssize_t rsize;
         char buf[65535];
         unsigned char *rbuf;
-        gnutls_x509_crq crq;
+        gnutls_x509_crq_t crq;
         size_t size = sizeof(buf);
 
         prelude_log_debug(1, "Sending certificate request.\n");
@@ -823,11 +854,11 @@ int tls_request_certificate(prelude_client_profile_t *cp, prelude_io_t *fd,
 }
 
 
-static int crt_import(gnutls_x509_crt *crt, const char *filename)
+static int crt_import(gnutls_x509_crt_t *crt, const char *filename)
 {
         int ret;
         size_t dsize;
-        gnutls_datum data;
+        gnutls_datum_t data;
 
         ret = _prelude_load_file(filename, &data.data, &dsize);
         if ( ret < 0 ) {
@@ -848,7 +879,7 @@ static int crt_import(gnutls_x509_crt *crt, const char *filename)
 
 
 
-static int crt_export(prelude_client_profile_t *cp, gnutls_x509_crt *crt, const char *filename)
+static int crt_export(prelude_client_profile_t *cp, gnutls_x509_crt_t *crt, const char *filename)
 {
         int ret;
         size_t size;
@@ -874,7 +905,7 @@ static int crt_export(prelude_client_profile_t *cp, gnutls_x509_crt *crt, const
 }
 
 
-int tls_load_ca_certificate(prelude_client_profile_t *cp, gnutls_x509_privkey key, gnutls_x509_crt *crt)
+int tls_load_ca_certificate(prelude_client_profile_t *cp, gnutls_x509_privkey_t key, gnutls_x509_crt_t *crt)
 {
         int ret;
         char filename[256];
@@ -899,13 +930,13 @@ int tls_load_ca_certificate(prelude_client_profile_t *cp, gnutls_x509_privkey ke
 
 
 int tls_load_ca_signed_certificate(prelude_client_profile_t *cp,
-                                   gnutls_x509_privkey cakey,
-                                   gnutls_x509_crt cacrt,
-                                   gnutls_x509_crt *crt)
+                                   gnutls_x509_privkey_t cakey,
+                                   gnutls_x509_crt_t cacrt,
+                                   gnutls_x509_crt_t *crt)
 {
         int ret;
         char filename[256];
-        gnutls_x509_crq crq;
+        gnutls_x509_crq_t crq;
         unsigned char buf[65535];
         size_t size = sizeof(buf);
 
@@ -932,13 +963,16 @@ int tls_load_ca_signed_certificate(prelude_client_profile_t *cp,
 
 
 
-int tls_revoke_analyzer(prelude_client_profile_t *cp, gnutls_x509_privkey key,
-                        gnutls_x509_crt crt, uint64_t revoked_analyzerid)
+int tls_revoke_analyzer(prelude_client_profile_t *cp, gnutls_x509_privkey_t x509key,
+                        gnutls_x509_crt_t crt, uint64_t revoked_analyzerid)
 {
         int ret, i;
         size_t len, dsize;
-        gnutls_datum data;
-        gnutls_x509_crl crl;
+        gnutls_datum_t data;
+#ifdef HAVE_GNUTLS_PRIVKEY_ABSTRACT
+        gnutls_privkey_t key;
+#endif
+        gnutls_x509_crl_t crl;
         uint64_t analyzerid;
         char crlfile[PATH_MAX], buf[65535];
 
@@ -980,7 +1014,14 @@ int tls_revoke_analyzer(prelude_client_profile_t *cp, gnutls_x509_privkey key,
         gnutls_x509_crl_set_next_update(crl, time(NULL));
         gnutls_x509_crl_set_this_update(crl, time(NULL));
 
-        ret = gnutls_x509_crl_sign(crl, crt, key);
+#ifdef HAVE_GNUTLS_PRIVKEY_ABSTRACT
+        key = get_privkey_from_x509(x509key);
+        if ( ! key )
+                return -1;
+        ret = gnutls_x509_crl_privkey_sign(crl, crt, key, GNUTLS_DIG_SHA256, 0);
+#else
+        ret = gnutls_x509_crl_sign(crl, crt, x509key);
+#endif
         if ( ret < 0 ) {
                 fprintf(stderr, "error signing CRL: %s.\n", gnutls_strerror(ret));
                 return -1;
