@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2002-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2002-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -644,3 +644,39 @@ void _prelude_unload_file(unsigned char *fdata, size_t size)
         free(fdata);
 #endif
 }
+
+
+double prelude_simple_strtod(const char *s, char **endptr)
+{
+        double neg = 1, ret = 0, fp = 1.0;
+        int got_point = 0, d;
+
+        if ( *s == '-' ) {
+                neg = -1;
+                s++;
+        }
+
+        for ( ; *s; s++) {
+                if ( *s == '.' ) {
+                        got_point = 1;
+                        continue;
+                }
+
+                d = *s - '0';
+                if ( d < 0 || d > 9 )
+                        break;
+
+                else if ( ! got_point )
+                        ret = (10 * ret) + d;
+
+                else {
+                        fp *= 0.1;
+                        ret += fp * d;
+                }
+        }
+
+        if ( endptr )
+                *endptr = s;
+
+        return ret * neg;
+}
