@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -31,6 +31,7 @@
 
 #include "prelude-log.h"
 #include "prelude-string.h"
+#include "prelude-error.h"
 #include "prelude-inttypes.h"
 #include "idmef.h"
 #include "idmef-tree-wrap.h"
@@ -166,8 +167,8 @@ static const struct {
         { IDMEF_ADDITIONAL_DATA_TYPE_BYTE, IDMEF_DATA_TYPE_BYTE, sizeof(uint8_t) },
         { IDMEF_ADDITIONAL_DATA_TYPE_CHARACTER, IDMEF_DATA_TYPE_CHAR, sizeof(char) },
         { IDMEF_ADDITIONAL_DATA_TYPE_DATE_TIME, IDMEF_DATA_TYPE_CHAR_STRING, 0 },
-        { IDMEF_ADDITIONAL_DATA_TYPE_INTEGER, IDMEF_DATA_TYPE_UINT32, sizeof(uint32_t) },
-        { IDMEF_ADDITIONAL_DATA_TYPE_NTPSTAMP, IDMEF_DATA_TYPE_UINT64, sizeof(uint64_t) },
+        { IDMEF_ADDITIONAL_DATA_TYPE_INTEGER, IDMEF_DATA_TYPE_INT, sizeof(uint64_t) },
+        { IDMEF_ADDITIONAL_DATA_TYPE_NTPSTAMP, IDMEF_DATA_TYPE_INT, sizeof(uint64_t) },
         { IDMEF_ADDITIONAL_DATA_TYPE_PORTLIST, IDMEF_DATA_TYPE_CHAR_STRING, 0 },
         { IDMEF_ADDITIONAL_DATA_TYPE_REAL, IDMEF_DATA_TYPE_FLOAT, sizeof(float) },
         { IDMEF_ADDITIONAL_DATA_TYPE_BOOLEAN, IDMEF_DATA_TYPE_BYTE, sizeof(prelude_bool_t) },
@@ -180,19 +181,22 @@ static const struct {
 static int check_type(idmef_additional_data_type_t type, const unsigned char *buf, size_t len)
 {
         if ( type < 0 || (size_t) type >= sizeof(idmef_additional_data_type_table) / sizeof(*idmef_additional_data_type_table) )
-                return -1;
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Invalid IDMEFAdditionalData type specified");
 
         if ( idmef_additional_data_type_table[type].len != 0 &&
-             len != idmef_additional_data_type_table[type].len )
-                return -1;
+             len > idmef_additional_data_type_table[type].len )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Provided value length does not match specified type length");
 
         if ( idmef_additional_data_type_table[type].len == 0 && len < 1 )
-                return -1;
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Invalid value length for this type");
 
-        if ( type == IDMEF_ADDITIONAL_DATA_TYPE_BYTE_STRING )
-                return 0;
+        if ( type == IDMEF_ADDITIONAL_DATA_TYPE_STRING ||
+             type == IDMEF_ADDITIONAL_DATA_TYPE_DATE_TIME ||
+             type == IDMEF_ADDITIONAL_DATA_TYPE_PORTLIST ||
+             type == IDMEF_ADDITIONAL_DATA_TYPE_XML )
+                return buf[len - 1] == '\0' ? 0 : prelude_error_verbose(PRELUDE_ERROR_GENERIC, "String is not nul terminated");
 
-        return buf[len - 1] == '\0' ? 0 : -1;
+        return 0;
 }
 
 
@@ -350,7 +354,7 @@ int idmef_additional_data_set_ptr_nodup_fast(idmef_additional_data_t *data,
  * Declare stuff
  */
 IDMEF_ADDITIONAL_DATA_SIMPLE(IDMEF_DATA_TYPE_FLOAT, float, IDMEF_ADDITIONAL_DATA_TYPE_REAL, float, real)
-IDMEF_ADDITIONAL_DATA_SIMPLE(IDMEF_DATA_TYPE_UINT32, uint32, IDMEF_ADDITIONAL_DATA_TYPE_INTEGER, uint32_t, integer)
+IDMEF_ADDITIONAL_DATA_SIMPLE(IDMEF_DATA_TYPE_INT, int, IDMEF_ADDITIONAL_DATA_TYPE_INTEGER, uint32_t, integer)
 IDMEF_ADDITIONAL_DATA_SIMPLE(IDMEF_DATA_TYPE_BYTE, byte, IDMEF_ADDITIONAL_DATA_TYPE_BOOLEAN, prelude_bool_t, boolean)
 IDMEF_ADDITIONAL_DATA_SIMPLE(IDMEF_DATA_TYPE_BYTE, byte, IDMEF_ADDITIONAL_DATA_TYPE_BYTE, uint8_t, byte)
 IDMEF_ADDITIONAL_DATA_SIMPLE(IDMEF_DATA_TYPE_CHAR, char, IDMEF_ADDITIONAL_DATA_TYPE_CHARACTER, char, character)
@@ -426,8 +430,8 @@ int idmef_additional_data_data_to_string(idmef_additional_data_t *ad, prelude_st
         switch ( idmef_additional_data_get_type(ad) ) {
 
         case IDMEF_ADDITIONAL_DATA_TYPE_NTPSTAMP:
-                i = idmef_data_get_uint64(data);
-                ret = prelude_string_sprintf(out, "0x%08lux.0x%08lux", (unsigned long) (i >> 32), (unsigned long) i);
+                i = idmef_data_get_int(data);
+                ret = prelude_string_sprintf(out, "0x%" PRELUDE_PRIx32 ".0x%" PRELUDE_PRIx32, (uint32_t) (i >> 32), (uint32_t) i);
                 break;
 
         default:
