@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2002-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2002-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 * Author: Krzysztof Zaraska
 *
@@ -16,9 +16,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -45,6 +45,8 @@
 #include "idmef-class.h"
 
 #include "idmef-tree-wrap.h"
+#include "idmef-message-print.h"
+#include "idmef-message-print-json.h"
 #include "idmef-tree-data.h"
 #include "idmef-path.h"
 
@@ -95,12 +97,25 @@ idmef_class_child_id_t idmef_class_find_child(idmef_class_id_t class, const char
                                 return i;
         }
 
-        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_CLASS_UNKNOWN_CHILD, "Unknown IDMEF child '%s'", name);
+        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_CLASS_UNKNOWN_CHILD, "Unknown IDMEF child '%s' for class '%s'", name, idmef_class_get_name(class));
 }
 
 
 
 
+prelude_bool_t idmef_class_is_child_union_member(idmef_class_id_t class, idmef_class_child_id_t child)
+{
+        int ret;
+
+        ret = is_child_valid(class, child);
+        if ( ret < 0 )
+                return ret;
+
+        return (object_data[class].children_list[child].union_id > 0) ? TRUE : FALSE;
+}
+
+
+
 prelude_bool_t idmef_class_is_child_list(idmef_class_id_t class, idmef_class_child_id_t child)
 {
         int ret;
@@ -114,6 +129,18 @@ prelude_bool_t idmef_class_is_child_list(idmef_class_id_t class, idmef_class_chi
 
 
 
+prelude_bool_t idmef_class_is_child_keyed_list(idmef_class_id_t class, idmef_class_child_id_t child)
+{
+        int ret;
+
+        ret = is_child_valid(class, child);
+        if ( ret < 0 )
+                return ret;
+
+        return object_data[class].children_list[child].keyed_list;
+}
+
+
 
 idmef_value_type_id_t idmef_class_get_child_value_type(idmef_class_id_t class, idmef_class_child_id_t child)
 {
@@ -128,6 +155,36 @@ idmef_value_type_id_t idmef_class_get_child_value_type(idmef_class_id_t class, i
 
 
 
+size_t idmef_class_get_child_count(idmef_class_id_t class)
+{
+        int ret;
+
+        ret = is_class_valid(class);
+        if ( ret < 0 )
+                return ret;
+
+        return object_data[class].children_list_elem;
+}
+
+
+
+int idmef_class_get_child_union_id(idmef_class_id_t class, idmef_class_child_id_t child)
+{
+        int ret;
+        const children_list_t *c;
+
+        ret = is_child_valid(class, child);
+        if ( ret < 0 )
+                return ret;
+
+        c = &object_data[class].children_list[child];
+        if ( ! c->union_id )
+                return prelude_error(PRELUDE_ERROR_IDMEF_CLASS_CHILD_NOT_UNION);
+
+        return c->union_id;
+}
+
+
 
 idmef_class_id_t idmef_class_get_child_class(idmef_class_id_t class, idmef_class_child_id_t child)
 {
@@ -310,6 +367,32 @@ int idmef_class_ref(idmef_class_id_t class, void *obj)
 }
 
 
+
+int idmef_class_print(idmef_class_id_t class, void *obj, prelude_io_t *fd)
+{
+        int ret;
+
+        ret = is_class_valid(class);
+        if ( ret < 0 )
+                return ret;
+
+        return object_data[class].print(obj, fd);
+}
+
+
+
+int idmef_class_print_json(idmef_class_id_t class, void *obj, prelude_io_t *fd)
+{
+        int ret;
+
+        ret = is_class_valid(class);
+        if ( ret < 0 )
+                return ret;
+
+        return object_data[class].print_json(obj, fd);
+}
+
+
 const char *idmef_class_get_name(idmef_class_id_t class)
 {
         if ( is_class_valid(class) < 0 )
@@ -317,3 +400,16 @@ const char *idmef_class_get_name(idmef_class_id_t class)
 
         return object_data[class].name;
 }
+
+
+
+prelude_bool_t idmef_class_is_listed(idmef_class_id_t class)
+{
+        int ret;
+
+        ret = is_class_valid(class);
+        if ( ret < 0 )
+                return ret;
+
+        return object_data[class].is_listed;
+}
