@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -323,7 +323,7 @@ idmef_criterion_operator_t idmef_criterion_get_operator(const idmef_criterion_t
 /**
  * idmef_criterion_match:
  * @criterion: Pointer to a #idmef_criterion_t object.
- * @message: Pointer to a #idmef_message_t object to match against @criterion.
+ * @object: Pointer to a #idmef_object_t object to match against @criterion.
  *
  * Matches @message against the provided @criterion. This implies retrieving the
  * value associated with @criterion path, and matching it with the @idmef_criterion_value_t
@@ -331,33 +331,21 @@ idmef_criterion_operator_t idmef_criterion_get_operator(const idmef_criterion_t
  *
  * Returns: 1 for a match, 0 for no match, or a negative value if an error occured.
  */
-int idmef_criterion_match(const idmef_criterion_t *criterion, idmef_message_t *message)
+int idmef_criterion_match(const idmef_criterion_t *criterion, void *object)
 {
         int ret;
-        idmef_value_t *value;
+        idmef_value_t *value = NULL;
 
         prelude_return_val_if_fail(criterion, prelude_error(PRELUDE_ERROR_ASSERTION));
-        prelude_return_val_if_fail(message, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(object, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-
-        ret = idmef_path_get(criterion->path, message, &value);
+        ret = idmef_path_get(criterion->path, object, &value);
         if ( ret < 0 )
                 return ret;
 
-        if ( ret == 0 ) {
-                if ( criterion->value && criterion->operator & IDMEF_CRITERION_OPERATOR_NOT )
-                        return 1;
-
-                return (criterion->operator == IDMEF_CRITERION_OPERATOR_NULL) ? 1 : 0;
-        }
-
-        if ( ! criterion->value ) {
-                idmef_value_destroy(value);
-                return (criterion->operator == (IDMEF_CRITERION_OPERATOR_NULL|IDMEF_CRITERION_OPERATOR_NOT)) ? 1 : 0;
-        }
-
         ret = idmef_criterion_value_match(criterion->value, value, criterion->operator);
-        idmef_value_destroy(value);
+        if ( value )
+                idmef_value_destroy(value);
 
         if ( ret < 0 )
                 return ret;
@@ -640,28 +628,28 @@ void idmef_criteria_set_criterion(idmef_criteria_t *criteria, idmef_criterion_t
 /**
  * idmef_criteria_match:
  * @criteria: Pointer to a #idmef_criteria_t object.
- * @message: Pointer to a #idmef_message_t message.
+ * @object: Pointer to a #idmef_object_t object.
  *
- * Matches @message against the provided criteria.
+ * Matches @object against the provided criteria.
  *
  * Returns: 1 if criteria match, 0 if it did not, a negative value if an error occured.
  */
-int idmef_criteria_match(const idmef_criteria_t *criteria, idmef_message_t *message)
+int idmef_criteria_match(const idmef_criteria_t *criteria, void *object)
 {
         int ret;
 
         prelude_return_val_if_fail(criteria, prelude_error(PRELUDE_ERROR_ASSERTION));
-        prelude_return_val_if_fail(message, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(object, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        ret = idmef_criterion_match(criteria->criterion, message);
+        ret = idmef_criterion_match(criteria->criterion, object);
         if ( ret < 0 )
                 return ret;
 
         if ( ret == 1 && criteria->and )
-                ret = idmef_criteria_match(criteria->and, message);
+                ret = idmef_criteria_match(criteria->and, object);
 
         if ( ret == 0 && criteria->or )
-                ret = idmef_criteria_match(criteria->or, message);
+                ret = idmef_criteria_match(criteria->or, object);
 
         if ( ret < 0 )
                 return ret;
