@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 * Author: Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
@@ -16,9 +16,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -160,10 +160,29 @@ static int btime_parse_wday(const char *value, int *out)
 }
 
 
+
+static int integer_compare(time_t matched, time_t wanted, idmef_criterion_operator_t op)
+{
+        if ( op & IDMEF_CRITERION_OPERATOR_EQUAL && matched == wanted )
+                return 1;
+
+        if ( op & IDMEF_CRITERION_OPERATOR_LESSER && matched < wanted)
+                return 1;
+
+        if ( op & IDMEF_CRITERION_OPERATOR_GREATER && matched > wanted )
+                return 1;
+
+        return 0;
+}
+
+
+
 static int do_btime_match(const idmef_criterion_value_t *cv, idmef_criterion_operator_t op, idmef_value_t *value)
 {
-        time_t sec, wanted, matched;
-        struct tm lt, *comp = cv->value;
+        int ret;
+        time_t sec;
+        struct tm lt, comp = *(struct tm *) cv->value;
+        prelude_bool_t need_full_compare = FALSE;
 
         if ( idmef_value_get_type(value) != IDMEF_VALUE_TYPE_TIME )
                 return -1;
@@ -175,28 +194,47 @@ static int do_btime_match(const idmef_criterion_value_t *cv, idmef_criterion_ope
         /*
          * Apply mask
          */
-        if ( comp->tm_sec < 0 ) lt.tm_sec = -1;
-        if ( comp->tm_min < 0 ) lt.tm_min = -1;
-        if ( comp->tm_mon < 0 ) lt.tm_mon = -1;
-        if ( comp->tm_hour < 0 ) lt.tm_hour = -1;
-        if ( comp->tm_mday < 0 ) lt.tm_mday = -1;
-        if ( comp->tm_year < 0 ) lt.tm_year = -1;
-        if ( comp->tm_wday < 0 ) lt.tm_wday = -1;
-        if ( comp->tm_yday < 0 ) lt.tm_yday = -1;
+        if ( comp.tm_sec < 0 ) lt.tm_sec = -1;
+        else need_full_compare = TRUE;
 
-        wanted  = timegm(comp);
-        matched = timegm(&lt);
+        if ( comp.tm_min < 0 ) lt.tm_min = -1;
+        else need_full_compare = TRUE;
 
-        if ( op & IDMEF_CRITERION_OPERATOR_EQUAL && matched == wanted )
+        if ( comp.tm_mon < 0 ) lt.tm_mon = -1;
+        else need_full_compare = TRUE;
+
+        if ( comp.tm_hour < 0 ) lt.tm_hour = -1;
+        else need_full_compare = TRUE;
+
+        if ( comp.tm_mday < 0 ) lt.tm_mday = -1;
+        else need_full_compare = TRUE;
+
+        if ( comp.tm_year < 0 ) lt.tm_year = -1;
+        else need_full_compare = TRUE;
+
+        if ( comp.tm_wday < 0 ) lt.tm_wday = -1;
+        if ( comp.tm_yday < 0 ) lt.tm_yday = -1;
+
+        /*
+         * The timegm() function ignores values supplied in the tm_wday
+         * and tm_yday fields, match them manually:
+         */
+        if ( comp.tm_wday >= 0 ) {
+                ret = integer_compare(lt.tm_wday, comp.tm_wday, (need_full_compare) ? op | IDMEF_CRITERION_OPERATOR_EQUAL : op);
+                if ( ret != 1 )
+                        return ret;
+        }
+
+        if ( comp.tm_yday >= 0 ) {
+                ret = integer_compare(lt.tm_yday, comp.tm_yday, (need_full_compare) ? op | IDMEF_CRITERION_OPERATOR_EQUAL : op);
+                if ( ret != 1 )
+                        return ret;
+        }
+
+        if ( ! need_full_compare )
                 return 1;
 
-        if ( op & IDMEF_CRITERION_OPERATOR_LESSER && matched < wanted)
-                return 1;
-
-        if ( op & IDMEF_CRITERION_OPERATOR_GREATER && matched > wanted )
-                return 1;
-
-        return 0;
+        return integer_compare(timegm(&lt), timegm(&comp), op);
 }
 
 
@@ -433,12 +471,17 @@ static int do_match_cb(idmef_value_t *value, void *extra)
         idmef_criterion_value_t *cv = mcb->cv;
         idmef_criterion_operator_t operator = mcb->operator;
 
-        if ( idmef_value_is_list(value) )
-                return idmef_value_iterate(value, do_match_cb, mcb);
+        if ( ! cv )
+                ret = (operator == IDMEF_CRITERION_OPERATOR_NOT_NULL) ? (value != NULL) : (value == NULL);
 
-        ret = cv->match(cv, operator, value);
-        if ( ret < 0 )
-                return ret;
+        else {
+                if ( value && idmef_value_is_list(value) )
+                        return idmef_value_iterate(value, do_match_cb, mcb);
+
+                ret = cv->match(cv, operator, value);
+                if ( ret < 0 )
+                        return ret;
+        }
 
         if ( ret > 0 )
                 mcb->match++;
@@ -497,6 +540,16 @@ int idmef_criterion_value_to_string(idmef_criterion_value_t *cv, prelude_string_
 
 
 
+static int my_value_match(idmef_value_t *value, struct match_cb *mcb)
+{
+        if ( value )
+                return idmef_value_iterate(value, do_match_cb, mcb);
+        else
+                return do_match_cb(value, mcb);
+}
+
+
+
 int idmef_criterion_value_match(idmef_criterion_value_t *cv, idmef_value_t *value,
                                 idmef_criterion_operator_t op)
 {
@@ -507,19 +560,17 @@ int idmef_criterion_value_match(idmef_criterion_value_t *cv, idmef_value_t *valu
         mcb.operator = op;
         mcb.match = mcb.nmatch = 0;
 
-        ret = idmef_value_iterate(value, do_match_cb, &mcb);
+        ret = my_value_match(value, &mcb);
         if ( ret < 0 )
                 return ret;
 
-        /*
-         * When comparing a path to a value using a negative operator, if the
-         * excluded value was found at least once, return a negative match.
-         *
-         * This is required since the 'match' value might also be positive
-         * in case the path contain a list of value.
-         */
-        if ( op & IDMEF_CRITERION_OPERATOR_NOT && mcb.nmatch > 0 )
-                return 0;
+        if ( value && idmef_value_is_list(value) && op != IDMEF_CRITERION_OPERATOR_NOT_NULL ) {
+                if ( op == IDMEF_CRITERION_OPERATOR_NULL )
+                        return (mcb.nmatch == 0) ? 1 : 0;
+
+                if ( op & IDMEF_CRITERION_OPERATOR_NOT && mcb.nmatch > 0 )
+                        return 0;
+        }
 
         return mcb.match;
 }
