@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2016 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -46,7 +46,7 @@
 #define IDMEF_DATA_OWN_DATA       0x2
 
 
-#define IDMEF_DATA_DECL(idmef_data_type, c_type, name)                \
+#define IDMEF_DATA_DECL(idmef_data_type, c_type, name, fname)         \
 int idmef_data_new_ ## name(idmef_data_t **nd, c_type val)            \
 {                                                                     \
         int ret;                                                      \
@@ -55,7 +55,7 @@ int idmef_data_new_ ## name(idmef_data_t **nd, c_type val)            \
         if ( ret < 0 )                                                \
                 return ret;                                           \
                                                                       \
-        idmef_data_set_ ## name(*nd, val);                            \
+        idmef_data_set_ ## fname(*nd, val);                           \
                                                                       \
         return ret;                                                   \
 }                                                                     \
@@ -66,20 +66,21 @@ void idmef_data_set_ ## name(idmef_data_t *ptr, c_type val)           \
         idmef_data_destroy_internal(ptr);                             \
         ptr->type = idmef_data_type;                                  \
         ptr->len = sizeof(val);                                       \
-        ptr->data.name ## _data = val;                                \
+        ptr->data.fname ## _data = val;                               \
 }                                                                     \
                                                                       \
 c_type idmef_data_get_ ## name(const idmef_data_t *ptr)               \
 {                                                                     \
-        return ptr->data.name ## _data;                               \
+        return ptr->data.fname ## _data;                              \
 }
 
 
-IDMEF_DATA_DECL(IDMEF_DATA_TYPE_CHAR, char, char)
-IDMEF_DATA_DECL(IDMEF_DATA_TYPE_BYTE, uint8_t, byte)
-IDMEF_DATA_DECL(IDMEF_DATA_TYPE_UINT32, uint32_t, uint32)
-IDMEF_DATA_DECL(IDMEF_DATA_TYPE_UINT64, uint64_t, uint64)
-IDMEF_DATA_DECL(IDMEF_DATA_TYPE_FLOAT, float, float)
+IDMEF_DATA_DECL(IDMEF_DATA_TYPE_CHAR, char, char, char)
+IDMEF_DATA_DECL(IDMEF_DATA_TYPE_BYTE, uint8_t, byte, byte)
+IDMEF_DATA_DECL(IDMEF_DATA_TYPE_INT, uint32_t, uint32, int)
+IDMEF_DATA_DECL(IDMEF_DATA_TYPE_INT, uint64_t, uint64, int)
+IDMEF_DATA_DECL(IDMEF_DATA_TYPE_INT, int64_t, int, int)
+IDMEF_DATA_DECL(IDMEF_DATA_TYPE_FLOAT, float, float, float)
 
 
 int idmef_data_new(idmef_data_t **data)
@@ -95,6 +96,30 @@ int idmef_data_new(idmef_data_t **data)
 }
 
 
+int idmef_data_new_time(idmef_data_t **data, idmef_time_t *time)
+{
+        int ret;
+
+        ret = idmef_data_new(data);
+        if ( ret < 0 )
+                return ret;
+
+        (*data)->len = 1;
+        (*data)->type = IDMEF_DATA_TYPE_TIME;
+        (*data)->flags |= IDMEF_DATA_OWN_DATA;
+        (*data)->data.ro_data = time;
+
+        return 0;
+}
+
+
+void idmef_data_set_time(idmef_data_t *data, idmef_time_t *time)
+{
+        data->len = 1;
+        data->type = IDMEF_DATA_TYPE_TIME;
+        data->flags |= IDMEF_DATA_OWN_DATA;
+        data->data.ro_data = time;
+}
 
 idmef_data_t *idmef_data_ref(idmef_data_t *data)
 {
@@ -255,6 +280,8 @@ int idmef_data_copy_ref(const idmef_data_t *src, idmef_data_t *dst)
  */
 int idmef_data_copy_dup(const idmef_data_t *src, idmef_data_t *dst)
 {
+        int ret;
+
         prelude_return_val_if_fail(src, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(dst, prelude_error(PRELUDE_ERROR_ASSERTION));
 
@@ -264,7 +291,13 @@ int idmef_data_copy_dup(const idmef_data_t *src, idmef_data_t *dst)
         dst->flags |= IDMEF_DATA_OWN_DATA;
         dst->len = src->len;
 
-        if ( src->type == IDMEF_DATA_TYPE_CHAR_STRING || src->type == IDMEF_DATA_TYPE_BYTE_STRING ) {
+        if ( src->type == IDMEF_DATA_TYPE_TIME ) {
+                ret = idmef_time_clone((idmef_time_t *) src->data.rw_data, (idmef_time_t **) &dst->data.rw_data);
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        else if ( src->type == IDMEF_DATA_TYPE_CHAR_STRING || src->type == IDMEF_DATA_TYPE_BYTE_STRING ) {
                 dst->data.rw_data = malloc(src->len);
                 if ( ! dst->data.rw_data )
                         return -1;
@@ -358,7 +391,9 @@ const void *idmef_data_get_data(const idmef_data_t *data)
         case IDMEF_DATA_TYPE_UNKNOWN:
                 return NULL;
 
-        case IDMEF_DATA_TYPE_CHAR_STRING: case IDMEF_DATA_TYPE_BYTE_STRING:
+        case IDMEF_DATA_TYPE_CHAR_STRING:
+        case IDMEF_DATA_TYPE_BYTE_STRING:
+        case IDMEF_DATA_TYPE_TIME:
                 return data->data.ro_data;
 
         default:
@@ -455,11 +490,8 @@ int idmef_data_to_string(const idmef_data_t *data, prelude_string_t *out)
                 break;
 
         case IDMEF_DATA_TYPE_UINT32:
-                ret = prelude_string_sprintf(out, "%u", data->data.uint32_data);
-                break;
-
-        case IDMEF_DATA_TYPE_UINT64:
-                ret = prelude_string_sprintf(out, "%" PRELUDE_PRIu64, data->data.uint64_data);
+        case IDMEF_DATA_TYPE_INT:
+                ret = prelude_string_sprintf(out, "%" PRELUDE_PRId64, data->data.int_data);
                 break;
 
         case IDMEF_DATA_TYPE_FLOAT:
@@ -467,12 +499,17 @@ int idmef_data_to_string(const idmef_data_t *data, prelude_string_t *out)
                 break;
 
         case IDMEF_DATA_TYPE_CHAR_STRING:
-                ret = prelude_string_sprintf(out, "%s", (const char *) data->data.ro_data);
+                if ( data->len )
+                        ret = prelude_string_ncat(out, (const char *) data->data.ro_data, data->len - 1);
                 break;
 
         case IDMEF_DATA_TYPE_BYTE_STRING:
                 ret = bytes_to_string(out, data->data.ro_data, data->len);
                 break;
+
+        case IDMEF_DATA_TYPE_TIME:
+                ret = idmef_time_to_string(data->data.ro_data, out);
+                break;
         }
 
         return ret;
@@ -488,7 +525,10 @@ void idmef_data_destroy_internal(idmef_data_t *ptr)
 {
         prelude_return_if_fail(ptr);
 
-        if ( (ptr->type == IDMEF_DATA_TYPE_CHAR_STRING || ptr->type == IDMEF_DATA_TYPE_BYTE_STRING) &&
+        if ( ptr->type == IDMEF_DATA_TYPE_TIME && ptr->flags & IDMEF_DATA_OWN_DATA )
+                idmef_time_destroy(ptr->data.rw_data);
+
+        else if ( (ptr->type == IDMEF_DATA_TYPE_CHAR_STRING || ptr->type == IDMEF_DATA_TYPE_BYTE_STRING) &&
              ptr->flags & IDMEF_DATA_OWN_DATA ) {
                 free(ptr->data.rw_data);
                 ptr->data.rw_data = NULL;
@@ -651,8 +691,12 @@ int idmef_data_compare(const idmef_data_t *data1, const idmef_data_t *data2)
         else if ( data1->type != data2->type )
                 return -1;
 
-        if ( data1->type == IDMEF_DATA_TYPE_CHAR_STRING || data1->type == IDMEF_DATA_TYPE_BYTE_STRING )
+        if ( data1->type == IDMEF_DATA_TYPE_TIME )
+                return idmef_time_compare(data1->data.ro_data, data2->data.ro_data);
+
+        else if ( data1->type == IDMEF_DATA_TYPE_CHAR_STRING || data1->type == IDMEF_DATA_TYPE_BYTE_STRING )
                 return memcmp(data1->data.ro_data, data2->data.ro_data, data1->len);
+
         else
                 return memcmp(&data1->data.char_data, &data2->data.char_data, data1->len);
 }
