@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2002-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2002-2016 CS-SI. All Rights Reserved.
 * Author: Krzysztof Zaraska
 *         Yoann Vandoorselaere <yoann@prelude-ids.com>
 *
@@ -16,9 +16,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -47,6 +47,7 @@
 #include "idmef-class.h"
 #include "idmef-value.h"
 
+#include "idmef-object-prv.h"
 #include "idmef-tree-wrap.h"
 #include "idmef-path.h"
 #include "prelude-string.h"
@@ -59,12 +60,21 @@
 #define HASH_DEFAULT_SIZE       128
 
 #define INDEX_UNDEFINED INT_MIN
-#define INDEX_FORBIDDEN (INT_MIN + 1)
+#define INDEX_KEY       (INT_MIN + 1)
+#define INDEX_FORBIDDEN (INT_MIN + 2)
+
+
+typedef struct idmef_key_listed_object {
+        IDMEF_OBJECT;
+        prelude_list_t list;
+        prelude_string_t *objkey;
+} idmef_key_listed_object_t;
 
 
 typedef struct idmef_path_element {
 
         int index;
+        char *index_key;
 
         idmef_class_id_t class;
         idmef_class_child_id_t position;
@@ -79,20 +89,15 @@ struct idmef_path {
         char name[MAX_NAME_LEN];
         int refcount;
         unsigned int depth;
+        idmef_class_id_t top_class;
 
         idmef_path_element_t elem[MAX_DEPTH];
 };
 
 
-typedef struct {
-        const idmef_path_t *path;
-        idmef_message_t *message;
-        prelude_bool_t reversed;
-        prelude_bool_t delete_list;
-} value_list_t;
 
-
-static int do_idmef_value_iterate(idmef_value_t *value, value_list_t *vl);
+static int _idmef_path_set(const idmef_path_t *path, idmef_class_id_t class, size_t i,
+                           int index_override, void *ptr, idmef_value_t *value);
 
 
 static prelude_bool_t flush_cache = FALSE;
@@ -131,15 +136,39 @@ static void flush_cache_if_wanted(void *ptr)
 
 
 
-/*
- * call with mutex held.
- */
+static unsigned int path_hash_func(const void *path)
+{
+        const char *ptr = ((const idmef_path_t *) path)->name;
+        unsigned int hv = *ptr;
+
+        if ( hv )
+                for ( ptr += 1; *ptr; ptr++ )
+                        hv = (hv << 5) - hv + *ptr;
+
+        return hv;
+
+}
+
+
+static int path_key_cmp_func(const void *p1, const void *p2)
+{
+        const idmef_path_t *path1 = p1;
+        const idmef_path_t *path2 = p2;
+
+        if ( path1->top_class != path2->top_class )
+                return -1;
+
+        return strcmp(path1->name, path2->name);
+}
+
+
+
 static int initialize_path_cache_if_needed(void)
 {
         if ( cached_path )
                 return 0;
 
-        return prelude_hash_new2(&cached_path, HASH_DEFAULT_SIZE, NULL, NULL, NULL, flush_cache_if_wanted);
+        return prelude_hash_new2(&cached_path, HASH_DEFAULT_SIZE, path_hash_func, path_key_cmp_func, NULL, flush_cache_if_wanted);
 }
 
 
@@ -148,7 +177,6 @@ static int build_name(idmef_path_t *path)
 {
         unsigned int i;
         const char *name;
-        char buf[16] = { 0 };
         idmef_class_id_t class;
 
         /*
@@ -170,8 +198,15 @@ static int build_name(idmef_path_t *path)
                 strncat(path->name, name, sizeof(path->name) - strlen(path->name));
 
                 if ( path->elem[i].index != INDEX_UNDEFINED && path->elem[i].index != INDEX_FORBIDDEN ) {
-                        snprintf(buf, sizeof(buf), "(%d)", path->elem[i].index);
-                        strncat(path->name, buf, sizeof(path->name) - strlen(path->name));
+                        strncat(path->name, "(", sizeof(path->name) - strlen(path->name));
+
+                        if ( path->elem[i].index != INDEX_KEY )
+                                snprintf(path->name + strlen(path->name), sizeof(path->name) - strlen(path->name), "%d", path->elem[i].index);
+
+                        else if ( path->elem[i].index_key )
+                                strncat(path->name, path->elem[i].index_key, sizeof(path->name) - strlen(path->name));
+
+                        strncat(path->name, ")", sizeof(path->name) - strlen(path->name));
                 }
 
                 class = idmef_class_get_child_class(class, path->elem[i].position);
@@ -189,6 +224,28 @@ static int idmef_path_get_internal(idmef_value_t **ret, const idmef_path_t *path
                                    unsigned int depth, void *parent, idmef_class_id_t parent_class);
 
 
+static inline prelude_bool_t has_index_key(prelude_list_t *elem, const char *index_key)
+{
+        idmef_key_listed_object_t *obj = prelude_list_entry(elem, idmef_key_listed_object_t, list);
+        return (obj->objkey && strcmp(prelude_string_get_string_or_default(obj->objkey, ""), index_key) == 0) ? TRUE : FALSE;
+}
+
+
+static int set_index_key(const idmef_path_element_t *elem, void *ptr)
+{
+        idmef_key_listed_object_t *obj = ptr;
+
+        if ( ! elem->index_key )
+                return 0;
+
+        if ( obj->objkey )
+                prelude_string_destroy(obj->objkey);
+
+        return prelude_string_new_dup(&obj->objkey, elem->index_key);
+}
+
+
+
 static int idmef_path_get_list_internal(idmef_value_t **value_list,
                                         const idmef_path_t *path, int depth,
                                         prelude_list_t *list, idmef_class_id_t parent_class)
@@ -205,29 +262,27 @@ static int idmef_path_get_list_internal(idmef_value_t **value_list,
         prelude_list_for_each(list, tmp) {
                 value = NULL;
 
+                if ( path->elem[depth - 1].index_key && ! has_index_key(tmp, path->elem[depth - 1].index_key) )
+                        continue;
+
                 if ( parent_class >= 0 )
-                        ret = idmef_path_get_internal(&value, path, depth, tmp, parent_class);
+                        ret = idmef_path_get_internal(&value, path, depth, idmef_linked_object_get_object(tmp), parent_class);
                 else {
                         idmef_value_type_id_t type = path->elem[depth - 1].value_type;
 
-                        ret = idmef_value_new(&value, type, tmp);
+                        ret = idmef_value_new(&value, type, idmef_linked_object_get_object(tmp));
                         if ( ret == 0 ) {
                                 idmef_value_type_t vt;
 
                                 vt.id = type;
-                                vt.data.string_val = (void *) tmp;
+                                vt.data.string_val = (void *) idmef_linked_object_get_object(tmp);
 
                                 ret = idmef_value_type_ref(&vt);
                                 if ( ret < 0 )
                                         idmef_value_destroy(value);
-                                else
-                                        ret = 1;
                         }
                 }
 
-                if ( ret == 0 )
-                        continue;
-
                 if ( ret < 0 ) {
                         idmef_value_destroy(*value_list);
                         return ret;
@@ -242,25 +297,26 @@ static int idmef_path_get_list_internal(idmef_value_t **value_list,
                 cnt++;
         }
 
-        if ( ! cnt )
+        if ( ! cnt ) {
                 idmef_value_destroy(*value_list);
+                *value_list = NULL;
+        }
 
         return cnt;
 }
 
 
-
 static int idmef_path_get_nth_internal(idmef_value_t **value, const idmef_path_t *path,
                                        unsigned int depth, prelude_list_t *list,
-                                       idmef_class_id_t parent_class, int which)
+                                       idmef_class_id_t parent_class, int which, const char *index_key)
 {
         int cnt = 0;
-        prelude_list_t *tmp;
+        prelude_list_t *tmp = NULL;
 
         if ( which >= 0 ) {
                 prelude_list_for_each(list, tmp) {
                         if ( cnt++ == which )
-                                return idmef_path_get_internal(value, path, depth, tmp, parent_class);
+                                return idmef_path_get_internal(value, path, depth, idmef_linked_object_get_object(tmp), parent_class);
                 }
         } else {
                 which = -which;
@@ -268,7 +324,7 @@ static int idmef_path_get_nth_internal(idmef_value_t **value, const idmef_path_t
 
                 prelude_list_for_each_reversed(list, tmp) {
                         if ( cnt++ == which )
-                                return idmef_path_get_internal(value, path, depth, tmp, parent_class);
+                                return idmef_path_get_internal(value, path, depth, idmef_linked_object_get_object(tmp), parent_class);
                 }
         }
 
@@ -293,8 +349,10 @@ static int idmef_path_get_internal(idmef_value_t **value, const idmef_path_t *pa
                 if ( ret < 0 )
                         return ret;
 
-                if ( ! child )
+                if ( ! child ) {
+                        *value = NULL;
                         return 0;
+                }
 
                 child_class = idmef_class_get_child_class(parent_class, child_id);
                 which = path->elem[depth].index;
@@ -302,10 +360,10 @@ static int idmef_path_get_internal(idmef_value_t **value, const idmef_path_t *pa
                 if ( which == INDEX_FORBIDDEN )
                         return idmef_path_get_internal(value, path, depth + 1, child, child_class);
 
-                if ( which == INDEX_UNDEFINED || which == IDMEF_LIST_APPEND || which == IDMEF_LIST_PREPEND )
+                if ( which == INDEX_UNDEFINED || which == INDEX_KEY || which == IDMEF_LIST_APPEND || which == IDMEF_LIST_PREPEND )
                         return idmef_path_get_list_internal(value, path, depth + 1, child, child_class);
 
-                return idmef_path_get_nth_internal(value, path, depth + 1, child, child_class, which);
+                return idmef_path_get_nth_internal(value, path, depth + 1, child, child_class, which, path->elem[depth].index_key);
         }
 
         if ( parent_class < 0 || (path->depth > 0 && path->elem[path->depth - 1].value_type == IDMEF_VALUE_TYPE_ENUM) ) {
@@ -332,7 +390,10 @@ static void delete_listed_child(void *parent, idmef_class_id_t class, const idme
                 return;
 
         prelude_list_for_each_safe(head, tmp, bkp) {
-                obj = prelude_linked_object_get_object(tmp);
+                obj = idmef_linked_object_get_object(tmp);
+
+                if ( elem->index_key && ! has_index_key(obj, elem->index_key) )
+                        continue;
 
                 /*
                  * The object might be referenced from other place than
@@ -347,45 +408,139 @@ static void delete_listed_child(void *parent, idmef_class_id_t class, const idme
         }
 }
 
-static int _idmef_path_set(const idmef_path_t *path, idmef_message_t *message, idmef_value_t *value, prelude_bool_t *delete_list)
+
+
+static int _idmef_path_set_undefined_not_last(const idmef_path_t *path, const idmef_path_element_t *elem,
+                                              idmef_class_id_t class, size_t i, void *ptr, idmef_value_t *value)
 {
-        size_t i;
-        void *ptr;
-        int ret, index;
+        int j = 0, ret;
+        idmef_value_t *val;
+        prelude_list_t *head, *tmp, *bkp;
+
+        assert(idmef_class_is_child_list(class, elem->position));
+
+        ret = idmef_class_get_child(ptr, class, elem->position, (void *) &head);
+        if ( ret < 0 )
+                return ret;
+
+        prelude_list_for_each_safe(head, tmp, bkp) {
+                if ( elem->index_key && ! has_index_key(tmp, elem->index_key) )
+                        continue;
+
+                if ( value && idmef_value_is_list(value) ) {
+                        ret = idmef_value_get_nth2(value, j++, &val);
+                        if ( ret <= 0 )
+                                return ret;
+                } else {
+                        j = 1;
+                        val = value;
+                }
+
+                ret = _idmef_path_set(path, elem->class, i + 1, 0, idmef_linked_object_get_object(tmp), val);
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        for ( ; value && j < idmef_value_get_count(value); j++ ) {
+                ret = _idmef_path_set(path, class, i, IDMEF_LIST_APPEND, ptr, idmef_value_get_nth(value, j));
+                if ( ret < 0 )
+                        return ret;
+
+                ret = set_index_key(elem, idmef_linked_object_get_object(head->prev));
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        return ret;
+}
+
+
+
+static void ad_magic_set_datatype(idmef_additional_data_t *data, idmef_value_type_id_t vtype)
+{
+        idmef_additional_data_type_t type;
+
+        /*
+         * In case the user has already set the type member, do nothing.
+         */
+        if ( _idmef_additional_data_type_is_set(data) != 0 )
+                return;
+
+        if ( vtype >= IDMEF_VALUE_TYPE_INT8 && vtype <= IDMEF_VALUE_TYPE_UINT64 )
+                type = IDMEF_ADDITIONAL_DATA_TYPE_INTEGER;
+
+        else if ( vtype == IDMEF_VALUE_TYPE_FLOAT || vtype == IDMEF_VALUE_TYPE_DOUBLE )
+                type = IDMEF_ADDITIONAL_DATA_TYPE_REAL;
+
+        else if ( vtype == IDMEF_VALUE_TYPE_STRING )
+                type = IDMEF_ADDITIONAL_DATA_TYPE_STRING;
+
+        else if ( vtype == IDMEF_VALUE_TYPE_DATA )
+                type = IDMEF_ADDITIONAL_DATA_TYPE_BYTE_STRING;
+
+       else if ( vtype == IDMEF_VALUE_TYPE_TIME )
+                type = IDMEF_ADDITIONAL_DATA_TYPE_DATE_TIME;
+
+        else
+                return;
+
+        idmef_additional_data_set_type(data, type);
+}
+
+
+
+static int _idmef_path_set(const idmef_path_t *path, idmef_class_id_t class, size_t i,
+                           int index_override, void *ptr, idmef_value_t *value)
+{
+        void *prevptr = NULL;
+        int ret, index, j;
+        prelude_bool_t is_last_element;
         idmef_value_type_id_t tid;
         const idmef_path_element_t *elem;
-        idmef_class_id_t class, parent_class;
+        idmef_class_id_t parent_class;
 
-        ptr = message;
-        parent_class = class = IDMEF_CLASS_ID_MESSAGE;
+        parent_class = (class == IDMEF_CLASS_ID_MESSAGE) ? class : path->elem[i - 1].class;
+
+        for ( ; i < path->depth; i++ ) {
 
-        for ( i = 0; i < path->depth; i++ ) {
                 elem = &path->elem[i];
-                index = elem->index;
+                index = (index_override) ? index_override : elem->index;
+                index_override = 0;
+                is_last_element = (i == (path->depth - 1));
 
-                /*
-                 * We don't want to allow the user to set a single field within a listed
-                 * object without specifying the object index: alert.source.interface = "blah".
-                 */
-                if ( i + 1 < path->depth && index == INDEX_UNDEFINED && idmef_class_is_child_list(class, elem->position) )
-                        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_MISS_INDEX,
-                                                     "IDMEF path element '%s' need indexing",
-                                                     idmef_class_get_name(elem->class));
-
-                if ( index == INDEX_UNDEFINED && idmef_class_is_child_list(class, elem->position) ) {
-                        index = IDMEF_LIST_APPEND;
-                        if ( *delete_list ) {
-                                *delete_list = FALSE;
+                if ( index == INDEX_UNDEFINED || index == INDEX_KEY ) {
+                        if ( ! is_last_element )
+                                return _idmef_path_set_undefined_not_last(path, elem, class, i, ptr, value);
+                        else {
                                 delete_listed_child(ptr, class, elem);
 
-                                if ( ! value )
-                                        return 0;
+                                for ( j = 0; value && j < idmef_value_get_count(value); j++ ) {
+                                        ret = _idmef_path_set(path, class, i, IDMEF_LIST_APPEND, ptr, idmef_value_get_nth(value, j));
+                                        if ( ret < 0 )
+                                                return ret;
+
+                                        if ( index == INDEX_KEY ) {
+                                                prelude_list_t *head;
+
+                                                ret = idmef_class_get_child(ptr, class, elem->position, (void *) &head);
+                                                if ( ret < 0 )
+                                                        return ret;
+
+                                                ret = set_index_key(elem, head->prev);
+                                                if ( ret < 0 )
+                                                        return ret;
+                                        }
+                                }
+
+                                return 0;
                         }
                 }
 
                 parent_class = class;
 
-                if ( value || i < (path->depth - 1) ) {
+                if ( value || ! is_last_element ) {
+                        prevptr = ptr;
+
                         ret = idmef_class_new_child(ptr, class, elem->position, index, &ptr);
                         if ( ret < 0 )
                                 return ret;
@@ -395,46 +550,23 @@ static int _idmef_path_set(const idmef_path_t *path, idmef_message_t *message, i
                 }
         }
 
+        elem = &path->elem[path->depth - 1];
         if ( ! value )
-                return idmef_class_destroy_child(ptr, parent_class,
-                                                 path->elem[path->depth - 1].position,
-                                                 path->elem[path->depth - 1].index);
+                return idmef_class_destroy_child(ptr, parent_class, elem->position, elem->index);
+
+        tid = idmef_class_get_child_value_type(parent_class, elem->position);
+        if ( parent_class == IDMEF_CLASS_ID_ADDITIONAL_DATA && tid == IDMEF_VALUE_TYPE_DATA && prevptr )
+                ad_magic_set_datatype(prevptr, idmef_value_get_type(value));
 
-        tid = idmef_class_get_child_value_type(parent_class, path->elem[path->depth - 1].position);
         return _idmef_value_copy_internal(value, tid, class, ptr);
 }
 
 
-static int unroll_listed_value(idmef_value_t *value, void *extra)
-{
-        value_list_t *val = extra;
-
-        if ( idmef_value_is_list(value) )
-                return do_idmef_value_iterate(value, extra);
-
-        return _idmef_path_set(val->path, val->message, value, &val->delete_list);
-}
-
-
-static int do_idmef_value_iterate(idmef_value_t *value, value_list_t *vl)
-{
-        int ret;
-
-        if ( vl->reversed )
-                ret = idmef_value_iterate_reversed(value, unroll_listed_value, vl);
-        else
-                ret = idmef_value_iterate(value, unroll_listed_value, vl);
-
-        return ret;
-}
-
-
-
 
 /**
  * idmef_path_get:
  * @path: Pointer to a #idmef_path_t object.
- * @message: Pointer to a #idmef_message_t object.
+ * @obj: Pointer to a #idmef_object_t object.
  * @ret: Address where to store the retrieved #idmef_value_t.
  *
  * This function retrieves the value for @path within @message,
@@ -442,9 +574,15 @@ static int do_idmef_value_iterate(idmef_value_t *value, value_list_t *vl)
  *
  * Returns: The number of element retrieved, or a negative value if an error occured.
  */
-int idmef_path_get(const idmef_path_t *path, idmef_message_t *message, idmef_value_t **ret)
+int idmef_path_get(const idmef_path_t *path, void *obj, idmef_value_t **ret)
 {
-        return idmef_path_get_internal(ret, path, 0, message, IDMEF_CLASS_ID_MESSAGE);
+        idmef_object_t *object = obj;
+
+        if ( object->_idmef_object_id != path->top_class )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "path for object '%s' used with '%s' object root",
+                                             idmef_class_get_name(path->top_class), idmef_class_get_name(object->_idmef_object_id));
+
+        return idmef_path_get_internal(ret, path, 0, object, object->_idmef_object_id);
 }
 
 
@@ -452,37 +590,41 @@ int idmef_path_get(const idmef_path_t *path, idmef_message_t *message, idmef_val
 /**
  * idmef_path_set:
  * @path: Pointer to a #idmef_path_t object.
- * @message: Pointer to a #idmef_message_t object.
+ * @obj: Pointer to a #idmef_object_t object.
  * @value: Pointer to a #idmef_value_t object.
  *
  * This function sets the provided @value for @path within @message.
  *
  * Returns: 0 on success, a negative value if an error occured.
  */
-int idmef_path_set(const idmef_path_t *path, idmef_message_t *message, idmef_value_t *value)
+int idmef_path_set(const idmef_path_t *path, void *obj, idmef_value_t *value)
 {
-        prelude_bool_t delete_list = TRUE;
+        idmef_object_t *object = obj;
 
         if ( path->depth < 1 )
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Path with depth of 0 are not allowed");
 
-        /*
-         * Allow raw list copy (example: alert.source = alert.source,
-         * alert.source(>>) = alert.source, alert.source(<<) = alert.source.
-         */
-        if ( value && idmef_value_is_list(value) ) {
-                value_list_t vl;
-
-                vl.path = path;
-                vl.message = message;
-                vl.delete_list = TRUE;
-                vl.reversed = (path->elem[path->depth - 1].index == IDMEF_LIST_PREPEND) ? TRUE : FALSE;
-
-                return do_idmef_value_iterate(value, &vl);
-        }
+        if ( object->_idmef_object_id != path->top_class )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "path for object '%s' used with '%s' object root",
+                                             idmef_class_get_name(path->top_class), idmef_class_get_name(object->_idmef_object_id));
 
 
-        return _idmef_path_set(path, message, value, &delete_list);
+        return _idmef_path_set(path, object->_idmef_object_id, 0, 0, object, value);
+}
+
+
+
+static int copy_path_name(idmef_path_t *path, const char *buffer)
+{
+        size_t len;
+
+        len = strlen(buffer) + 1;
+        if ( len >= sizeof(path->name) )
+                return prelude_error(PRELUDE_ERROR_IDMEF_PATH_LENGTH);
+
+        memcpy(path->name, buffer, len);
+
+        return 0;
 }
 
 
@@ -493,9 +635,16 @@ int idmef_path_set(const idmef_path_t *path, idmef_message_t *message, idmef_val
  * 0 for a new empty object
  * 1 for an existing object already in the cache
  */
-static int idmef_path_create(idmef_path_t **path, const char *buffer)
+static int idmef_path_create(idmef_path_t **path, idmef_class_id_t rootclass, const char *buffer)
 {
         int ret;
+        idmef_path_t tmp;
+
+        ret = copy_path_name(&tmp, buffer);
+        if ( ret < 0 )
+                return ret;
+
+        tmp.top_class = rootclass;
 
         gl_lock_lock(cached_path_mutex);
 
@@ -505,7 +654,7 @@ static int idmef_path_create(idmef_path_t **path, const char *buffer)
                 return ret;
         }
 
-        *path = prelude_hash_get(cached_path, buffer);
+        *path = prelude_hash_get(cached_path, &tmp);
         gl_lock_unlock(cached_path_mutex);
 
         if ( *path )
@@ -549,26 +698,22 @@ static int parse_path_token(char **sptr, char **out)
 
 
 
-
-static int idmef_path_parse_new(idmef_path_t *path, const char *buffer)
+static int idmef_path_parse_new(idmef_path_t *path, idmef_class_id_t rootclass, const char *buffer)
 {
-        size_t len;
-        int index = -1, is_last;
+        int ret, index = -1, is_last;
         unsigned int depth = 0;
-        char *endptr, *ptr, *ptr2;
+        char *endptr, *ptr, *ptr2, *end;
         idmef_class_child_id_t child = 0;
-        idmef_class_id_t class, prev_class = 0;
+        idmef_class_id_t class;
         idmef_value_type_id_t vtype;
 
-        len = strlen(buffer) + 1;
-        if ( len >= sizeof(path->name) )
-                return prelude_error(PRELUDE_ERROR_IDMEF_PATH_LENGTH);
+        ret = copy_path_name(path, buffer);
+        if ( ret < 0 )
+                return ret;
 
-        memcpy(path->name, buffer, len);
-
-        ptr = NULL;
+        end = ptr = NULL;
         endptr = path->name;
-        class = IDMEF_CLASS_ID_MESSAGE;
+        path->top_class = class = rootclass;
 
         do {
                 index = INDEX_UNDEFINED;
@@ -576,15 +721,31 @@ static int idmef_path_parse_new(idmef_path_t *path, const char *buffer)
 
                 ptr2 = strchr(ptr, '(');
                 if ( ptr2 ) {
-                        *ptr2 = '\0';
-                        if ( strncmp(ptr2 + 1, "<<", 2) == 0 )
+                        char *errptr = NULL;
+                        char *tok = ptr2 + 1;
+
+                        end = strchr(ptr2, ')');
+                        if ( ! end )
+                                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Malformed IDMEFPath index : missing ')' character");
+
+                        *end = *ptr2 = '\0';
+                        if ( strcmp(tok, "<<") == 0 )
                                 index = IDMEF_LIST_PREPEND;
 
-                        else if ( strncmp(ptr2 + 1, ">>", 2) == 0 )
+                        else if ( strcmp(tok, ">>") == 0 )
                                 index = IDMEF_LIST_APPEND;
 
-                        else if ( strncmp(ptr2 + 1, "*", 1) != 0 )
-                                index = strtol(ptr2 + 1, NULL, 0);
+                        else if ( (*tok == '\'' || *tok == '"') && *(tok + strlen(tok) - 1) == *tok ) {
+                                index = INDEX_KEY;
+                                path->elem[depth].index_key = strndup(tok + 1, strlen(tok) - 2);
+                        }
+
+                        else if ( strcmp(ptr2 + 1, "*") != 0 ) {
+                                index = strtol(ptr2 + 1, &errptr, 0);
+                                if ( index == 0 && errptr == tok )
+                                        return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Invalid index specified : `%s`", tok);
+                        }
+                        *end = ')';
                 }
 
                 child = idmef_class_find_child(class, ptr);
@@ -603,11 +764,13 @@ static int idmef_path_parse_new(idmef_path_t *path, const char *buffer)
                                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN,
                                                              "Invalid IDMEF path element '%s': indexing not supported", ptr);
 
+                        if ( index == INDEX_KEY && ! idmef_class_is_child_keyed_list(class, child) )
+                                return prelude_error_verbose(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN,
+                                                             "Invalid IDMEF path element '%s': key not supported as index", ptr);
+
                         path->elem[depth].index = index;
                 }
 
-                prev_class = class;
-
                 /* The last object may not be a structure */
                 vtype = path->elem[depth].value_type = idmef_class_get_child_value_type(class, child);
                 if ( vtype != IDMEF_VALUE_TYPE_CLASS && ! is_last )
@@ -630,21 +793,23 @@ static int idmef_path_parse_new(idmef_path_t *path, const char *buffer)
 
 
 /**
- * idmef_path_new_fast:
+ * idmef_path_new_from_root_fast:
  * @path: Address where to store the created #idmef_path_t object.
+ * @rootclass: #idmef_class_id_t ID of the root path.
  * @buffer: Name of the path to create.
  *
- * Creates a #idmef_path_t object pointing to @buffer, and stores it within @path.
+ * Creates a #idmef_path_t object whom root is @rootclass pointing
+ * to @buffer, and stores it within @path.
  *
  * Returns: 0 on success, or a negative value if an error occured.
  */
-int idmef_path_new_fast(idmef_path_t **path, const char *buffer)
+int idmef_path_new_from_root_fast(idmef_path_t **path, idmef_class_id_t rootclass, const char *buffer)
 {
         int ret;
 
         prelude_return_val_if_fail(buffer, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        ret = idmef_path_create(path, buffer);
+        ret = idmef_path_create(path, rootclass, buffer);
         if ( ret < 0 )
                 return ret;
 
@@ -654,9 +819,9 @@ int idmef_path_new_fast(idmef_path_t **path, const char *buffer)
         }
 
         if ( *buffer == '\0' )
-                (*path)->elem[0].class = IDMEF_CLASS_ID_MESSAGE;
+                (*path)->top_class = rootclass;
         else {
-                ret = idmef_path_parse_new(*path, buffer);
+                ret = idmef_path_parse_new(*path, rootclass, buffer);
                 if ( ret < 0 ) {
                         gl_lock_destroy((*path)->mutex);
                         free(*path);
@@ -666,7 +831,7 @@ int idmef_path_new_fast(idmef_path_t **path, const char *buffer)
 
         gl_lock_lock(cached_path_mutex);
 
-        if ( prelude_hash_set(cached_path, (*path)->name, *path) < 0 ) {
+        if ( prelude_hash_add(cached_path, *path, *path) < 0 ) {
 
                 gl_lock_destroy((*path)->mutex);
                 free(*path);
@@ -683,6 +848,23 @@ int idmef_path_new_fast(idmef_path_t **path, const char *buffer)
 
 
 
+
+/**
+ * idmef_path_new_fast:
+ * @path: Address where to store the created #idmef_path_t object.
+ * @buffer: Name of the path to create.
+ *
+ * Creates a #idmef_path_t object pointing to @buffer, and stores it within @path.
+ *
+ * Returns: 0 on success, or a negative value if an error occured.
+ */
+int idmef_path_new_fast(idmef_path_t **path, const char *buffer)
+{
+        return idmef_path_new_from_root_fast(path, IDMEF_CLASS_ID_MESSAGE, buffer);
+}
+
+
+
 /**
  * idmef_path_new_v:
  * @path: Address where to store the created #idmef_path_t object.
@@ -755,7 +937,7 @@ idmef_class_id_t idmef_path_get_class(const idmef_path_t *path, int depth)
                 depth = path->depth - 1;
 
         if ( path->depth == 0 && depth < 0 )
-                return IDMEF_CLASS_ID_MESSAGE;
+                return path->top_class;
 
         return path->elem[depth].class;
 }
@@ -822,7 +1004,7 @@ static inline int invalidate(idmef_path_t *path)
 
         if ( path->refcount == 2 ) {
                 gl_lock_lock(cached_path_mutex);
-                ret = prelude_hash_elem_destroy(cached_path, path->name);
+                ret = prelude_hash_elem_destroy(cached_path, path);
                 gl_lock_unlock(cached_path_mutex);
 
                 if ( ret == 0 )
@@ -858,7 +1040,7 @@ int idmef_path_set_index(idmef_path_t *path, unsigned int depth, int index)
         prelude_return_val_if_fail(path, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(depth < path->depth, prelude_error(PRELUDE_ERROR_IDMEF_PATH_DEPTH));
 
-        if ( index == INDEX_FORBIDDEN )
+        if ( index == INDEX_FORBIDDEN || index == INDEX_KEY )
                 return prelude_error(PRELUDE_ERROR_IDMEF_PATH_INDEX_RESERVED);
 
         if ( path->elem[depth].index == INDEX_FORBIDDEN )
@@ -868,6 +1050,9 @@ int idmef_path_set_index(idmef_path_t *path, unsigned int depth, int index)
         if ( ret < 0 )
                 return ret;
 
+        if ( path->elem[depth].index == INDEX_KEY )
+                free(path->elem[depth].index_key);
+
         path->elem[depth].index = index;
 
         ret = build_name(path);
@@ -926,6 +1111,26 @@ int idmef_path_get_index(const idmef_path_t *path, unsigned int depth)
 
 
 
+int idmef_path_get_key(const idmef_path_t *path, unsigned int depth, const char **key)
+{
+        prelude_return_val_if_fail(path, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(depth < path->depth, prelude_error(PRELUDE_ERROR_IDMEF_PATH_DEPTH));
+
+        if ( path->elem[depth].index == INDEX_UNDEFINED )
+                return prelude_error(PRELUDE_ERROR_IDMEF_PATH_INDEX_UNDEFINED);
+
+        if ( path->elem[depth].index == INDEX_FORBIDDEN )
+                return prelude_error(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN);
+
+        if ( path->elem[depth].index != INDEX_KEY )
+                return prelude_error(PRELUDE_ERROR_IDMEF_PATH_INDEX_FORBIDDEN);
+
+        *key = path->elem[depth].index_key;
+        return 0;
+}
+
+
+
 /**
  * idmef_path_make_child:
  * @path: Pointer to an #idmef_path_t object.
@@ -1067,6 +1272,9 @@ int idmef_path_ncompare(const idmef_path_t *p1, const idmef_path_t *p2, unsigned
                 if ( p1->elem[i].index != p2->elem[i].index )
                         return -1;
 
+                if ( p1->elem[i].index == INDEX_KEY && strcmp(p1->elem[i].index_key, p2->elem[i].index_key) != 0 )
+                        return -1;
+
                 if ( p1->elem[i].position - p2->elem[i].position )
                         return -1;
         }
@@ -1110,18 +1318,33 @@ int idmef_path_compare(const idmef_path_t *p1, const idmef_path_t *p2)
  */
 int idmef_path_clone(const idmef_path_t *src, idmef_path_t **dst)
 {
+        unsigned int i;
+
         prelude_return_val_if_fail(src, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         *dst = calloc(1, sizeof(**dst));
         if ( ! *dst )
                 return prelude_error_from_errno(errno);
 
+        memcpy(*dst, src, sizeof(**dst));
+
+        for ( i = 0; i < src->depth; i++ ) {
+                if ( ! src->elem[i].index_key )
+                        continue;
+
+                (*dst)->elem[i].index_key = strdup(src->elem[i].index_key);
+                if ( ! (*dst)->elem[i].index_key ) {
+                        while ( --i ) {
+                                if ( (*dst)->elem[i].index_key )
+                                        free((*dst)->elem[i].index_key);
+                        }
+
+                        free(*dst);
+                        return prelude_error_from_errno(errno);
+                }
+        }
+
         (*dst)->refcount = 1;
-        (*dst)->depth = src->depth;
-
-        strncpy((*dst)->name, src->name, sizeof(src->name));
-        memcpy((*dst)->elem, src->elem, src->depth * sizeof(idmef_path_element_t));
-
         gl_lock_init((*dst)->mutex);
 
         return 0;
