@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2016 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -82,28 +82,50 @@ static char *parse_time_ymd(struct tm *tm, const char *buf)
 
 
 
-static char *parse_time_hmsu(struct tm *tm, uint32_t *usec, const char *buf)
+static int digit2usec(uint32_t n, int digit_count)
 {
-        char *ptr;
-        unsigned int fraction;
+        int i;
+        const size_t max_digit = 6; /* 999999 */
 
-        ptr = strptime(buf, "%H:%M:%S", tm);
-        if ( ! ptr )
-                return NULL;
+        if ( digit_count > max_digit )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "Invalid number of digits for time fraction");
 
-        if ( *ptr == '.' || *ptr == ',' ) {
-                ptr++;
+        for ( i = 0; i < (max_digit - digit_count); i++ )
+                n *= 10;
 
-                if ( sscanf(ptr, "%u", &fraction) < 1 )
-                        return NULL;
+        return n;
+}
 
-                *usec = fraction * 10000;
 
-                while ( isdigit((int) *ptr) )
-                        ptr++;
+
+static int parse_time_hmsu(struct tm *tm, uint32_t *usec, char **buf)
+{
+        int fraction;
+        char *eptr = NULL;
+
+        *buf = strptime(*buf, "%H:%M:%S", tm);
+        if ( ! *buf )
+                goto fmterror;
+
+        if ( **buf == '.' || **buf == ',' ) {
+                (*buf)++;
+
+                fraction = strtoul(*buf, &eptr, 10);
+                if ( eptr == *buf )
+                        goto fmterror;
+
+                fraction = digit2usec(fraction, eptr - *buf);
+                if ( fraction < 0 )
+                        return fraction;
+
+                *buf = eptr;
+                *usec = fraction;
         }
 
-        return ptr;
+        return 0;
+
+    fmterror:
+        return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "error parsing time field, format should be: HH:MM:SS(.fraction)");
 }
 
 
@@ -165,7 +187,7 @@ int idmef_time_set_from_string(idmef_time_t *time, const char *buf)
         int ret;
         char *ptr;
         struct tm tm;
-        prelude_bool_t miss_gmt = TRUE;
+        prelude_bool_t miss_gmt = FALSE;
 
         prelude_return_val_if_fail(time, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(buf, prelude_error(PRELUDE_ERROR_ASSERTION));
@@ -177,11 +199,15 @@ int idmef_time_set_from_string(idmef_time_t *time, const char *buf)
         if ( ! ptr )
                 return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "error parsing date field, format should be: YY-MM-DD");
 
-        if ( *ptr ) {
-                ptr = parse_time_hmsu(&tm, &time->usec, ptr);
-                if ( ! ptr )
-                        return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "error parsing time field, format should be: HH:MM:SS");
+        time->usec = 0;
+        time->gmt_offset = 0;
 
+        if ( *ptr ) {
+                ret = parse_time_hmsu(&tm, &time->usec, &ptr);
+                if ( ret < 0 )
+                        return ret;
+
+                miss_gmt = TRUE;
                 if ( *ptr ) {
                         ret = parse_time_gmt(&tm, &time->gmt_offset, ptr);
                         if ( ret < 0 )
@@ -332,7 +358,7 @@ int idmef_time_to_string(const idmef_time_t *time, prelude_string_t *out)
 {
         time_t t;
         struct tm utc;
-        uint32_t hour_off, min_off, sec_off;
+        uint32_t hour_off, min_off;
 
         prelude_return_val_if_fail(time, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(out, prelude_error(PRELUDE_ERROR_ASSERTION));
@@ -344,7 +370,6 @@ int idmef_time_to_string(const idmef_time_t *time, prelude_string_t *out)
 
         hour_off = time->gmt_offset / 3600;
         min_off = time->gmt_offset % 3600 / 60;
-        sec_off = time->gmt_offset % 60;
 
         return prelude_string_sprintf(out, "%d-%.2d-%.2dT%.2d:%.2d:%.2d.%02u%+.2d:%.2d",
                                       utc.tm_year + 1900, utc.tm_mon + 1, utc.tm_mday,
