@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -26,14 +26,18 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <assert.h>
 
+#include "common.h"
 #include "prelude-inttypes.h"
 #include "prelude-string.h"
 
 #define PRELUDE_ERROR_SOURCE_DEFAULT PRELUDE_ERROR_SOURCE_IDMEF_VALUE_TYPE
 #include "prelude-error.h"
 #include "prelude-inttypes.h"
+#include "prelude-linked-object.h"
+#include "idmef-object-prv.h"
 
 #include "idmef-time.h"
 #include "idmef-data.h"
@@ -58,38 +62,42 @@
 #define ENUM_OPERATOR    STRING_OPERATOR|INTEGER_OPERATOR
 
 
-#define GENERIC_ONE_BASE_RW_FUNC(scanfmt, printfmt, name, type)                          \
-        static int name ## _read(idmef_value_type_t *dst, const char *buf)               \
-        {                                                                                \
-                int ret;                                                                 \
-                ret = sscanf(buf, (scanfmt), &(dst)->data. name ##_val);                 \
-                return (ret == 1) ? 0 : prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE, \
-                                                              "Reading " #name " value failed");    \
-        }                                                                                \
-                                                                                         \
-        static int name ## _write(const idmef_value_type_t *src, prelude_string_t *out)  \
-        {                                                                                \
-                return prelude_string_sprintf(out, (printfmt), src->data.name ##_val);   \
+#define GENERIC_ONE_BASE_RW_FUNC(scanfmt, printfmt, name, type)                            \
+        static int name ## _read(idmef_value_type_t *dst, const char *buf)                 \
+        {                                                                                  \
+                char *endptr;                                                              \
+                                                                                           \
+                (dst)->data. name ##_val = strtod(buf, &endptr);                           \
+                if ( buf == endptr || *endptr != '\0' || errno == ERANGE )                 \
+                        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE, \
+                        "Reading " #name " value failed with '%s'", buf);                  \
+                return 1;                                                                  \
+        }                                                                                  \
+                                                                                           \
+        static int name ## _write(const idmef_value_type_t *src, prelude_string_t *out)    \
+        {                                                                                  \
+                return prelude_string_sprintf(out, (printfmt), src->data.name ##_val);     \
         }
 
 
-#define GENERIC_TWO_BASES_RW_FUNC(fmt_dec, fmt_hex, name, type)                         \
-        static int name ## _read(idmef_value_type_t *dst, const char *buf)              \
-        {                                                                               \
-                int ret;                                                                \
-                                                                                        \
-                if ( strncasecmp(buf, "0x", 2) == 0 )                                   \
-                        ret = sscanf(buf, (fmt_hex), &(dst)->data. name ##_val);        \
-                else                                                                    \
-                        ret = sscanf(buf, (fmt_dec), &(dst)->data. name ##_val);        \
-                                                                                        \
-                return (ret == 1) ? 0 : prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE, \
-                                                              "Reading " #name " value failed");    \
-        }                                                                               \
-                                                                                        \
-        static int name ## _write(const idmef_value_type_t *src, prelude_string_t *out) \
-        {                                                                               \
-                return prelude_string_sprintf(out, (fmt_dec), src->data.name ##_val);   \
+#define GENERIC_TWO_BASES_RW_FUNC(rfunc, fmt_dec, name, type, min, max)                              \
+        static int name ## _read(idmef_value_type_t *dst, const char *buf)                           \
+        {                                                                                            \
+                type tmp;                                                                            \
+                char *endptr;                                                                        \
+                                                                                                     \
+                tmp = rfunc(buf, &endptr, 0);                                                        \
+                if ( buf == endptr || *endptr != '\0' || tmp < min || tmp > max || errno == ERANGE ) \
+                        return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE,           \
+                        "Value out of range, required: [" # min " - " # max "], got '%s'", buf);     \
+                                                                                                     \
+                dst->data.name ##_val = tmp;                                                         \
+                return 0;                                                                            \
+        }                                                                                            \
+                                                                                                     \
+        static int name ## _write(const idmef_value_type_t *src, prelude_string_t *out)              \
+        {                                                                                            \
+                return prelude_string_sprintf(out, (fmt_dec), src->data.name ##_val);                \
         }
 
 
@@ -115,51 +123,17 @@ typedef struct {
 
 
 
-static int byte_read(idmef_value_type_t *dst, const char *buf, unsigned int min, unsigned int max)
-{
-        char *endptr;
-        long int tmp;
-
-        tmp = strtol(buf, &endptr, 0);
-        if ( buf == endptr || tmp < min || tmp > max )
-                return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_PARSE,
-                                             "Value out of range, required: [%u-%u], got %s",
-                                             min, max, buf);
-
-        dst->data.int8_val = (int8_t) tmp;
-
-        return 0;
-}
-
-
-static int int8_write(const idmef_value_type_t *src, prelude_string_t *out)
-{
-        return prelude_string_sprintf(out, "%d", (int) src->data.int8_val);
-}
-
-static int uint8_write(const idmef_value_type_t *src, prelude_string_t *out)
-{
-        return prelude_string_sprintf(out, "%u", (int) src->data.int8_val);
-}
-
-static int int8_read(idmef_value_type_t *dst, const char *buf)
-{
-        return byte_read(dst, buf, PRELUDE_INT8_MIN, PRELUDE_INT8_MAX);
-}
-
-static int uint8_read(idmef_value_type_t *dst, const char *buf)
-{
-        return byte_read(dst, buf, 0, PRELUDE_UINT8_MAX);
-}
-
-
-
-GENERIC_TWO_BASES_RW_FUNC("%hd", "%hx", int16, int16_t)
-GENERIC_TWO_BASES_RW_FUNC("%hu", "%hx", uint16, uint16_t)
-GENERIC_TWO_BASES_RW_FUNC("%d", "%x", int32, int32_t)
-GENERIC_TWO_BASES_RW_FUNC("%u", "%x", uint32, uint32_t)
-GENERIC_TWO_BASES_RW_FUNC("%" PRELUDE_PRId64, "%" PRELUDE_PRIx64, int64, int64_t)
-GENERIC_TWO_BASES_RW_FUNC("%" PRELUDE_PRIu64, "%" PRELUDE_PRIx64, uint64, uint64_t)
+/*
+ * We specify a type bigger than the one handled, in order to catch min/max error.
+ */
+GENERIC_TWO_BASES_RW_FUNC(strtol, "%d", int8, int, PRELUDE_INT8_MIN, PRELUDE_INT8_MAX)
+GENERIC_TWO_BASES_RW_FUNC(strtol, "%d", uint8, unsigned int, 0, PRELUDE_UINT8_MAX)
+GENERIC_TWO_BASES_RW_FUNC(strtol, "%hd", int16, int, PRELUDE_INT16_MIN, PRELUDE_INT16_MAX)
+GENERIC_TWO_BASES_RW_FUNC(strtoul, "%hu", uint16, unsigned int, 0, PRELUDE_UINT16_MAX)
+GENERIC_TWO_BASES_RW_FUNC(strtol, "%d", int32, int64_t, PRELUDE_INT32_MIN, PRELUDE_INT32_MAX)
+GENERIC_TWO_BASES_RW_FUNC(strtoul, "%u", uint32, uint64_t, 0, PRELUDE_UINT32_MAX)
+GENERIC_TWO_BASES_RW_FUNC(strtoll, "%" PRELUDE_PRId64, int64, int64_t, PRELUDE_INT64_MIN, PRELUDE_INT64_MAX)
+GENERIC_TWO_BASES_RW_FUNC(strtoull, "%" PRELUDE_PRIu64, uint64, uint64_t, 0, PRELUDE_UINT64_MAX)
 
 GENERIC_ONE_BASE_RW_FUNC("%f", "%f", float, float)
 GENERIC_ONE_BASE_RW_FUNC("%lf", "%f", double, double)
@@ -172,8 +146,8 @@ GENERIC_ONE_BASE_RW_FUNC("%lf", "%f", double, double)
  */
 static int charstring_compare(const char *s1, const char *s2, idmef_criterion_operator_t op)
 {
-        if ( ! s1 || ! s2 )
-                return (s1) ? 1 : -1;
+        if ( (s1 && !s2) || (s2 && !s1) )
+                return -1;
 
         if ( op == (IDMEF_CRITERION_OPERATOR_EQUAL|IDMEF_CRITERION_OPERATOR_NOCASE) && strcasecmp(s1, s2) == 0 )
                 return 0;
@@ -214,6 +188,9 @@ static int generic_compare(const idmef_value_type_t *t1, const idmef_value_type_
 {
         int ret;
 
+        if ( (t1 && ! t2) || (t2 && ! t1) )
+                return -1;
+
         ret = memcmp(&t1->data, &t2->data, size);
 
         if ( ret == 0 && op & IDMEF_CRITERION_OPERATOR_EQUAL )
@@ -267,6 +244,9 @@ static int enum_compare(const idmef_value_type_t *src, const idmef_value_type_t
 {
         const char *s1;
 
+        if ( (src && ! dst) || (dst && ! src) )
+                return -1;
+
         if ( dst->id == IDMEF_VALUE_TYPE_STRING ) {
                 s1 = idmef_class_enum_to_string(src->data.enum_val.class_id, src->data.enum_val.value);
                 if ( ! s1 )
@@ -287,6 +267,9 @@ static int time_compare(const idmef_value_type_t *t1, const idmef_value_type_t *
 {
         int ret;
 
+        if ( (t1 && ! t2) || (t2 && ! t1) )
+                return -1;
+
         ret = idmef_time_compare(t1->data.time_val, t2->data.time_val);
         if ( op & IDMEF_CRITERION_OPERATOR_EQUAL && ret == 0 )
                 return 0;
@@ -362,10 +345,10 @@ static int string_compare(const idmef_value_type_t *t1, const idmef_value_type_t
 {
         const char *s1 = NULL, *s2 = NULL;
 
-        if ( t1->data.string_val )
+        if ( t1 && t1->data.string_val )
                 s1 = prelude_string_get_string(t1->data.string_val);
 
-        if ( t2->data.string_val )
+        if ( t2 && t2->data.string_val )
                 s2 = prelude_string_get_string(t2->data.string_val);
 
         return charstring_compare(s1, s2, op);
@@ -409,7 +392,7 @@ static void string_destroy(idmef_value_type_t *type)
 static int string_write(const idmef_value_type_t *src, prelude_string_t *out)
 {
         return prelude_string_sprintf(out, "%s",
-                                      prelude_string_get_string(src->data.string_val));
+                       prelude_string_get_string_or_default(src->data.string_val, "<empty>"));
 }
 
 
@@ -424,14 +407,14 @@ static int data_compare(const idmef_value_type_t *t1, const idmef_value_type_t *
         size_t s1_len, s2_len;
         const void *s1 = NULL, *s2 = NULL;
 
-        if ( t1->data.data_val )
+        if ( t1 && t1->data.data_val )
                 s1 = idmef_data_get_data(t1->data.data_val);
 
-        if ( t2->data.string_val )
+        if ( t2 && t2->data.string_val )
                 s2 = idmef_data_get_data(t2->data.data_val);
 
-        if ( ! s1 || ! s2 )
-                return (s1) ? 1 : -1;
+        if ( (s1 && !s2) || (s2 && !s1) )
+                return -1;
 
         if ( op & IDMEF_CRITERION_OPERATOR_SUBSTR ) {
                 s1_len = idmef_data_get_len(t1->data.data_val);
@@ -497,11 +480,14 @@ static void data_destroy(idmef_value_type_t *type)
 
 
 /*
- *
+ * class specific
  */
 static int class_compare(const idmef_value_type_t *c1,
                          const idmef_value_type_t *c2, size_t len, idmef_criterion_operator_t op)
 {
+        if ( (c1 && !c2) || (c2 && !c1) )
+                return -1;
+
         return idmef_class_compare(c1->data.class_val.class_id,
                                    c1->data.class_val.object, c2->data.class_val.object);
 }
@@ -532,6 +518,133 @@ static void class_destroy(idmef_value_type_t *type)
 }
 
 
+
+static int list_to_string(const char *cname, idmef_class_id_t class, prelude_list_t *head, prelude_string_t *out,
+                          int (*item_to_string_cb)(int class, void *item, prelude_string_t *out))
+{
+        void *obj;
+        int ret, j;
+        prelude_list_t *tmp;
+
+        if ( prelude_list_is_empty(head) )
+                return 0;
+
+        if ( cname )
+                ret = prelude_string_sprintf(out, " %s:", cname);
+        else
+                ret = prelude_string_cat(out, " ");
+
+        if ( ret < 0 )
+                return ret;
+
+        ret = prelude_string_cat(out, "(");
+        if ( ret < 0 )
+                return ret;
+
+        j = 0;
+        prelude_list_for_each(head, tmp) {
+                if ( j++ > 0 ) {
+                        ret = prelude_string_cat(out, ", ");
+                        if ( ret < 0 )
+                                return ret;
+                }
+
+                if ( ! cname )
+                        obj = idmef_linked_object_get_object(tmp);
+                else
+                        obj = prelude_linked_object_get_object(tmp);
+
+                ret = item_to_string_cb(class, obj, out);
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        return prelude_string_cat(out, ")");
+}
+
+
+
+static int listed_str_to_string(idmef_class_id_t parent_class, void *parent, prelude_string_t *out)
+{
+        return prelude_string_cat(out, prelude_string_get_string(parent));
+}
+
+
+static int class_to_string(idmef_class_id_t parent_class, void *parent, prelude_string_t *out)
+{
+        int ret, i;
+        void *childptr;
+        idmef_class_id_t childclass;
+        idmef_value_type_id_t vtype;
+        const char *pname = idmef_class_get_name(parent_class);
+
+        ret = prelude_string_sprintf(out, "<IDMEF%c%s",  toupper(*pname), pname + 1);
+        if ( ret < 0 )
+                return ret;
+
+        for ( i = 0; ; i++ ) {
+                ret = idmef_class_get_child(parent, parent_class, i, &childptr);
+                if ( ret < 0 )
+                        break;
+
+                if ( ! childptr )
+                        continue;
+
+                vtype = idmef_class_get_child_value_type(parent_class, i);
+                childclass = idmef_class_get_child_class(parent_class, i);
+
+                if ( vtype != IDMEF_VALUE_TYPE_CLASS ) {
+                        const char *cname = idmef_class_get_child_name(parent_class, i);
+
+                        if ( ! idmef_class_is_child_list(parent_class, i) ) {
+                                ret = prelude_string_sprintf(out, " %s:", cname);
+                                if ( ret < 0 ) {
+                                        idmef_value_destroy(childptr);
+                                        return ret;
+                                }
+
+                                ret = idmef_value_to_string(childptr, out);
+                                idmef_value_destroy(childptr);
+                                if ( ret < 0 )
+                                        return ret;
+                        }
+
+                        else {
+                                ret = list_to_string(cname, childclass, childptr, out, listed_str_to_string);
+                                if ( ret < 0 )
+                                        return ret;
+                        }
+
+                        continue;
+                }
+
+                if ( ! idmef_class_is_child_list(parent_class, i) ) {
+                        ret = prelude_string_cat(out, " ");
+                        if ( ret < 0 )
+                                return ret;
+
+                        ret = class_to_string(childclass, childptr, out);
+                        if ( ret < 0 )
+                                return ret;
+                }
+
+                else {
+                        ret = list_to_string(NULL, childclass, childptr, out, class_to_string);
+                        if ( ret < 0 )
+                                return ret;
+                }
+        }
+
+        return prelude_string_cat(out, ">");
+}
+
+
+static int class_write(const idmef_value_type_t *src, prelude_string_t *out)
+{
+        return class_to_string(src->data.class_val.class_id, src->data.class_val.object, out);
+}
+
+
 static const idmef_value_type_operation_t ops_tbl[] = {
         { "unknown", 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL                     },
         { "int8", sizeof(int8_t), INTEGER_OPERATOR, generic_copy,
@@ -564,7 +677,7 @@ static const idmef_value_type_operation_t ops_tbl[] = {
           generic_clone, NULL, NULL, enum_compare, enum_read, enum_write,               },
         { "list", 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL                        },
         { "class", 0, CLASS_OPERATOR, class_copy, class_clone,
-          class_ref, class_destroy, class_compare, NULL, NULL                   },
+          class_ref, class_destroy, class_compare, NULL, class_write                    },
 };
 
 
@@ -653,24 +766,31 @@ int idmef_value_type_compare(const idmef_value_type_t *type1,
                              idmef_criterion_operator_t op)
 {
         int ret;
+        idmef_value_type_id_t tid;
 
-        ret = is_type_valid(type1->id);
+        prelude_return_val_if_fail(type1 || type2, prelude_error(PRELUDE_ERROR_ASSERTION));
+        tid = (type1) ? type1->id : type2->id;
+
+        ret = is_type_valid(tid);
         if ( ret < 0 )
                 return ret;
 
-        if ( type1->id != type2->id ) {
+        if ( type1 && type2 && type1->id != type2->id ) {
                 if ( type1->id != IDMEF_VALUE_TYPE_ENUM && type2->id != IDMEF_VALUE_TYPE_STRING )
                         return prelude_error(PRELUDE_ERROR_IDMEF_VALUE_TYPE_COMPARE_MISMATCH);
         }
 
-        assert(op & ops_tbl[type1->id].operator);
+        if ( ! (op & ops_tbl[tid].operator) )
+                return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_COMPARE_UNAVAILABLE,
+                                             "Object type '%s' does not support operator '%s'",
+                                             idmef_value_type_to_string(tid), idmef_criterion_operator_to_string(op));
 
-        if ( ! ops_tbl[type1->id].compare )
+        if ( ! ops_tbl[tid].compare )
                 return prelude_error_verbose(PRELUDE_ERROR_IDMEF_VALUE_TYPE_COMPARE_UNAVAILABLE,
                                              "Object type '%s' does not support compare operation",
-                                             idmef_value_type_to_string(type1->id));
+                                             idmef_value_type_to_string(tid));
 
-        ret = ops_tbl[type1->id].compare(type1, type2, ops_tbl[type1->id].len, op & ~IDMEF_CRITERION_OPERATOR_NOT);
+        ret = ops_tbl[tid].compare(type1, type2, ops_tbl[tid].len, op & ~IDMEF_CRITERION_OPERATOR_NOT);
         if ( ret < 0 ) /* not an error -> no match */
                 ret = 1;
 
