@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2016 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
@@ -16,9 +16,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -405,7 +405,6 @@ int idmef_value_new_list(idmef_value_t **value)
 int idmef_value_list_add(idmef_value_t *list, idmef_value_t *item)
 {
         prelude_return_val_if_fail(list, prelude_error(PRELUDE_ERROR_ASSERTION));
-        prelude_return_val_if_fail(item, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         if ( list->list_elems == list->list_max ) {
 
@@ -513,8 +512,10 @@ static int idmef_value_set_own_data(idmef_value_t *value, prelude_bool_t own_dat
         if ( ! value->list )
                 value->own_data = own_data;
 
-        else for ( cnt = 0 ; cnt < value->list_elems; cnt++ )
-                idmef_value_set_own_data(value->list[cnt], own_data);
+        else for ( cnt = 0 ; cnt < value->list_elems; cnt++ ) {
+                if ( value->list[cnt] )
+                        idmef_value_set_own_data(value->list[cnt], own_data);
+        }
 
         return 0;
 }
@@ -627,18 +628,44 @@ int idmef_value_iterate_reversed(const idmef_value_t *value,
 
 
 
-idmef_value_t *idmef_value_get_nth(const idmef_value_t *val, int n)
+int idmef_value_get_nth2(const idmef_value_t *val, int index, idmef_value_t **ret)
 {
-        prelude_return_val_if_fail(val, NULL);
+        prelude_return_val_if_fail(val, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(index <= val->list_elems, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        if ( ! val->list )
-                return (n == 0) ? value_ro2rw(val) : NULL;
+        if ( index == 0 && ! val->list ) {
+                *ret = value_ro2rw(val);
+                return 1;
+        }
 
-        return (n >= 0 && n < val->list_elems) ? val->list[n] : NULL;
+        else if ( index >= 0 && index < val->list_elems ) {
+                *ret = val->list[index];
+                return 1;
+        }
+
+        else if ( index >= val->list_elems )
+                return 0;
+
+        else return -1;
 }
 
 
 
+idmef_value_t *idmef_value_get_nth(const idmef_value_t *val, int n)
+{
+        int ret;
+        idmef_value_t *rv;
+
+        prelude_return_val_if_fail(val, NULL);
+
+        ret = idmef_value_get_nth2(val, n, &rv);
+        if ( ret != 1 )
+                return NULL;
+
+        return rv;
+}
+
+
 
 int idmef_value_get_count(const idmef_value_t *val)
 {
@@ -662,11 +689,17 @@ static int idmef_value_list_clone(const idmef_value_t *val, idmef_value_t **dst)
         (*dst)->list = malloc(((*dst)->list_elems + 1) * sizeof((*dst)->list));
 
         for ( cnt = 0; cnt < (*dst)->list_elems; cnt++ ) {
+                if ( ! val->list[cnt] ) {
+                        (*dst)->list[cnt] = NULL;
+                        continue;
+                }
 
                 ret = idmef_value_clone(val->list[cnt], &((*dst)->list[cnt]));
                 if ( ret < 0 ) {
-                        while ( --cnt >= 0 )
-                                idmef_value_destroy((*dst)->list[cnt]);
+                        while ( --cnt >= 0 ) {
+                                if ( (*dst)->list[cnt] )
+                                        idmef_value_destroy((*dst)->list[cnt]);
+                        }
                 }
 
                 free((*dst)->list);
@@ -713,12 +746,55 @@ idmef_value_t *idmef_value_ref(idmef_value_t *val)
 
 
 
-int idmef_value_to_string(const idmef_value_t *val, prelude_string_t *out)
+int idmef_value_to_string(const idmef_value_t *value, prelude_string_t *out)
 {
-        prelude_return_val_if_fail(val, prelude_error(PRELUDE_ERROR_ASSERTION));
+        int i, ret;
+        idmef_value_t *val;
+
+        prelude_return_val_if_fail(value, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(out, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        return idmef_value_type_write(&val->type, out);
+        if ( idmef_value_is_list(value) ) {
+                ret = prelude_string_cat(out, "(");
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        for ( i = 0; i < idmef_value_get_count(value); i++ ) {
+                val = idmef_value_get_nth(value, i);
+
+                if ( idmef_value_is_list(val) ) {
+                        ret = prelude_string_cat(out, "(");
+                        if ( ret < 0 )
+                                return ret;
+
+                        ret = idmef_value_to_string(val, out);
+                        if ( ret < 0 )
+                                return ret;
+
+                        ret = prelude_string_cat(out, ")");
+                        if ( ret < 0 )
+                                return ret;
+                } else {
+                        if ( i ) {
+                                ret = prelude_string_cat(out, ", ");
+                                if ( ret < 0 )
+                                        return ret;
+                        }
+
+                        ret = idmef_value_type_write(&val->type, out);
+                        if ( ret < 0 )
+                                return ret;
+                }
+        }
+
+        if ( idmef_value_is_list(value) ) {
+                ret = prelude_string_cat(out, ")");
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        return 0;
 }
 
 
@@ -761,7 +837,7 @@ static int idmef_value_match_internal(idmef_value_t *val1, void *extra)
         int ret;
         compare_t *compare = extra;
 
-        if ( idmef_value_is_list(val1) )
+        if ( val1 && idmef_value_is_list(val1) )
                 ret = idmef_value_iterate(val1, idmef_value_match_internal, extra);
 
         else if ( compare->val2 && idmef_value_is_list(compare->val2) ) {
@@ -773,7 +849,7 @@ static int idmef_value_match_internal(idmef_value_t *val1, void *extra)
         }
 
         else {
-                ret = idmef_value_type_compare(&val1->type, &compare->val2->type, compare->operator);
+                ret = idmef_value_type_compare((val1) ? &val1->type : NULL, &compare->val2->type, compare->operator);
                 if ( ret == 0 )
                         compare->match++;
         }
@@ -798,14 +874,17 @@ int idmef_value_match(idmef_value_t *val1, idmef_value_t *val2, idmef_criterion_
         int ret;
         compare_t compare;
 
-        prelude_return_val_if_fail(val1, prelude_error(PRELUDE_ERROR_ASSERTION));
-        prelude_return_val_if_fail(val2, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(val1 || val2, prelude_error(PRELUDE_ERROR_ASSERTION));
 
         compare.match = 0;
         compare.val2 = val2;
         compare.operator = op;
 
-        ret = idmef_value_iterate(val1, idmef_value_match_internal, &compare);
+        if ( val1 )
+                ret = idmef_value_iterate(val1, idmef_value_match_internal, &compare);
+        else
+                ret = idmef_value_match_internal(val1, &compare);
+
         if ( ret < 0 )
                 return ret;
 
@@ -866,8 +945,10 @@ void idmef_value_destroy(idmef_value_t *val)
                 return;
 
         if ( val->list ) {
-                for ( i = 0; i < val->list_elems; i++ )
-                        idmef_value_destroy(val->list[i]);
+                for ( i = 0; i < val->list_elems; i++ ) {
+                        if ( val->list[i] )
+                                idmef_value_destroy(val->list[i]);
+                }
 
                 free(val->list);
         }
@@ -899,7 +980,7 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_INT8 ) {
-                ret = idmef_data_new_uint32(&data, idmef_value_get_int8(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_int8(input));
                 if ( ret < 0 )
                         return ret;
 
@@ -907,7 +988,7 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_UINT8 ) {
-                ret = idmef_data_new_uint32(&data, idmef_value_get_uint8(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_uint8(input));
                 if ( ret < 0 )
                         return ret;
 
@@ -915,7 +996,7 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_INT16 ) {
-                ret = idmef_data_new_uint32(&data, idmef_value_get_int16(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_int16(input));
                 if ( ret < 0 )
                         return ret;
 
@@ -923,7 +1004,7 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_UINT16 ) {
-                ret = idmef_data_new_uint32(&data, idmef_value_get_uint16(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_uint16(input));
                 if ( ret < 0 )
                         return ret;
 
@@ -931,7 +1012,7 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_INT32 ) {
-                ret = idmef_data_new_uint32(&data, idmef_value_get_int32(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_int32(input));
                 if ( ret < 0 )
                         return ret;
 
@@ -939,7 +1020,7 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_UINT32 ) {
-                ret = idmef_data_new_uint32(&data, idmef_value_get_uint32(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_uint32(input));
                 if ( ret < 0 )
                         return ret;
 
@@ -947,7 +1028,7 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_INT64 ) {
-                ret = idmef_data_new_uint64(&data, idmef_value_get_int64(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_int64(input));
                 if ( ret < 0 )
                         return ret;
 
@@ -955,7 +1036,15 @@ static int cast_to_data(idmef_value_t *input)
         }
 
         else if ( vtype == IDMEF_VALUE_TYPE_UINT64 ) {
-                ret = idmef_data_new_uint64(&data, idmef_value_get_uint64(input));
+                ret = idmef_data_new_int(&data, idmef_value_get_uint64(input));
+                if ( ret < 0 )
+                        return ret;
+
+                return idmef_value_set_data(input, data);
+        }
+
+        else if ( vtype == IDMEF_VALUE_TYPE_TIME ) {
+                ret = idmef_data_new_time(&data, idmef_time_ref(idmef_value_get_time(input)));
                 if ( ret < 0 )
                         return ret;
 
@@ -966,7 +1055,7 @@ static int cast_to_data(idmef_value_t *input)
                 int64_t v = idmef_value_get_float(input);
 
                 if ( v == idmef_value_get_float(input) )
-                        ret = idmef_data_new_uint64(&data, v);
+                        ret = idmef_data_new_int(&data, v);
                 else
                         ret = idmef_data_new_float(&data, idmef_value_get_float(input));
 
@@ -980,7 +1069,7 @@ static int cast_to_data(idmef_value_t *input)
                 int64_t v = idmef_value_get_double(input);
 
                 if ( v == idmef_value_get_double(input) )
-                        ret = idmef_data_new_uint64(&data, v);
+                        ret = idmef_data_new_int(&data, v);
                 else
                         ret = idmef_data_new_float(&data, idmef_value_get_double(input));
 
@@ -990,7 +1079,9 @@ static int cast_to_data(idmef_value_t *input)
                 return idmef_value_set_data(input, data);
         }
 
-        return -1;
+        return prelude_error_verbose(PRELUDE_ERROR_GENERIC,
+                                     "Unable to cast input type '%s' to 'data'",
+                                     idmef_value_type_to_string(vtype));
 }
 
 
@@ -1185,6 +1276,8 @@ int _idmef_value_copy_internal(const idmef_value_t *val,
                         return ret;
 
                 ret = idmef_value_type_copy(&copy.type, res);
+                if ( val->own_data )
+                        idmef_value_type_destroy(&copy.type);
         }
 
         return ret;
