@@ -1,7 +1,7 @@
 /*****
 *
 *
-* Copyright (C) 2002-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2002-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 * Author: Krzysztof Zaraska <kzaraska@student.uci.agh.edu.pl>
 *
@@ -17,9 +17,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -30,19 +30,6 @@
  extern "C" {
 #endif
 
-#ifndef __attribute__
-/* This feature is available in gcc versions 2.5 and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
-#  define __attribute__(Spec) /* empty */
-# endif
-/* The __-protected variants of `format' and `printf' attributes
-   are accepted by gcc versions 2.6.4 (effectively 2.7) and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#  define __format__ format
-#  define __printf__ printf
-# endif
-#endif
-
 #ifdef HAVE_CONFIG_H
 # include "config.h"
 #endif
@@ -50,18 +37,19 @@
 typedef struct idmef_path idmef_path_t;
 
 #include <stdarg.h>
+#include "prelude-macros.h"
 #include "idmef-value.h"
 #include "idmef-tree-wrap.h"
 
-int idmef_path_get(const idmef_path_t *path, idmef_message_t *message, idmef_value_t **ret);
+int idmef_path_get(const idmef_path_t *path, void *object, idmef_value_t **ret);
 
-int idmef_path_set(const idmef_path_t *path, idmef_message_t *message, idmef_value_t *value);
+int idmef_path_set(const idmef_path_t *path, void *object, idmef_value_t *value);
 
-int idmef_path_new(idmef_path_t **path, const char *format, ...)
-                   __attribute__ ((__format__ (__printf__, 2, 3)));
+int idmef_path_new(idmef_path_t **path, const char *format, ...) PRELUDE_FMT_PRINTF(2, 3);
 
-int idmef_path_new_v(idmef_path_t **path, const char *format, va_list args)
-                     __attribute__ ((__format__ (__printf__, 2, 0)));
+int idmef_path_new_v(idmef_path_t **path, const char *format, va_list args) PRELUDE_FMT_PRINTF(2, 0);
+
+int idmef_path_new_from_root_fast(idmef_path_t **path, idmef_class_id_t rootclass, const char *buffer);
 
 int idmef_path_new_fast(idmef_path_t **path, const char *buffer);
 
@@ -75,6 +63,8 @@ int idmef_path_undefine_index(idmef_path_t *path, unsigned int depth);
 
 int idmef_path_get_index(const idmef_path_t *path, unsigned int depth);
 
+int idmef_path_get_key(const idmef_path_t *path, unsigned int depth, const char **key);
+
 int idmef_path_make_child(idmef_path_t *path, const char *child_name, int index);
 
 int idmef_path_make_parent(idmef_path_t *path);
