@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2002-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2002-2016 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 * Author: Krzysztof Zaraska <kzaraska@student.uci.agh.edu.pl>
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
@@ -17,9 +17,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -112,6 +112,8 @@ int idmef_value_iterate(const idmef_value_t *value, int (*callback)(idmef_value_
 
 int idmef_value_iterate_reversed(const idmef_value_t *value, int (*callback)(idmef_value_t *ptr, void *extra), void *extra);
 
+int idmef_value_get_nth2(const idmef_value_t *val, int index, idmef_value_t **ret);
+
 idmef_value_t *idmef_value_get_nth(const idmef_value_t *val, int n);
 
 int idmef_value_get_count(const idmef_value_t *val);
@@ -122,7 +124,7 @@ idmef_value_t *idmef_value_ref(idmef_value_t *val);
 
 int idmef_value_print(const idmef_value_t *val, prelude_io_t *fd);
 
-int idmef_value_to_string(const idmef_value_t *val, prelude_string_t *out);
+int idmef_value_to_string(const idmef_value_t *value, prelude_string_t *out);
 
 int idmef_value_get(const idmef_value_t *val, void *res);
 
