@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -28,9 +28,9 @@
 #include "prelude-list.h"
 
 
-#define PRELUDE_LINKED_OBJECT   \
-        prelude_list_t _list;   \
-        unsigned int _object_id
+#define PRELUDE_LINKED_OBJECT    \
+        unsigned int _object_id; \
+        prelude_list_t _list
 
 
 typedef struct {
