@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2005-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2005-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -30,27 +30,13 @@
 # include "config.h"
 #endif
 
+#include "prelude-macros.h"
 #include <stdarg.h>
 
 #ifdef __cplusplus
  extern "C" {
 #endif
 
-
-#ifndef __attribute__
-/* This feature is available in gcc versions 2.5 and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
-#  define __attribute__(Spec) /* empty */
-# endif
-/* The __-protected variants of `format' and `printf' attributes
-   are accepted by gcc versions 2.6.4 (effectively 2.7) and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#  define __format__ format
-#  define __printf__ printf
-# endif
-#endif
-
-
 typedef enum {
         PRELUDE_LOG_CRIT  = -1,
         PRELUDE_LOG_ERR   =  0,
@@ -69,11 +55,11 @@ typedef enum {
 
 void _prelude_log_v(prelude_log_t level, const char *file,
                     const char *function, int line, const char *fmt, va_list ap)
-                    __attribute__ ((__format__ (__printf__, 5, 0)));
+                    PRELUDE_FMT_PRINTF(5, 0);
 
 void _prelude_log(prelude_log_t level, const char *file,
                   const char *function, int line, const char *fmt, ...)
-                  __attribute__ ((__format__ (__printf__, 5, 6)));
+                  PRELUDE_FMT_PRINTF(5, 6);
 
 
 #ifdef HAVE_VARIADIC_MACROS
@@ -86,10 +72,10 @@ void _prelude_log(prelude_log_t level, const char *file,
 #else
 
 void prelude_log(prelude_log_t level, const char *fmt, ...)
-                 __attribute__ ((__format__ (__printf__, 2, 3)));
+                 PRELUDE_FMT_PRINTF(2, 3);
 
 void prelude_log_debug(prelude_log_t level, const char *fmt, ...)
-                       __attribute__ ((__format__ (__printf__, 2, 3)));
+                       PRELUDE_FMT_PRINTF(2, 3);
 
 #endif
 
