@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -65,6 +65,8 @@ int prelude_msg_set(prelude_msg_t *msg, uint8_t tag, uint32_t len, const void *d
 
 int prelude_msg_write(prelude_msg_t *msg, prelude_io_t *dst);
 
+int prelude_msg_write_r(prelude_msg_t *msg, prelude_io_t *dst, uint32_t *windex);
+
 
 
 /*
@@ -96,6 +98,8 @@ void prelude_msg_set_callback(prelude_msg_t *msg, int (*flush_msg_cb)(prelude_ms
 
 void prelude_msg_set_data(prelude_msg_t *msg, void *data);
 
+void *prelude_msg_get_data(prelude_msg_t *msg);
+
 prelude_msg_t *prelude_msg_ref(prelude_msg_t *msg);
 
 #ifdef __cplusplus
