@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -63,7 +63,19 @@ typedef struct {
 # define lt_preloaded_symbols lt__PROGRAM__LTX_preloaded_symbols
 #endif
 
+/*
+ * LT_DLSYM_CONST has been defined during the libtool 2.2.10 -> 2.4 
+ * transition. Use it to guess whether lt_preloaded_symbols is already
+ * defined by ltdl.h as a different kind of symbol.
+ * 
+ * We use void * so that application requiring plugin preloading doesn't 
+ * depend on libltdl headers and lib. In case the application doesn't 
+ * use libltdl, plugin preloading can still be used throught libprelude 
+ * preloading function.
+ */
+#ifndef LT_DLSYM_CONST
 extern const void *lt_preloaded_symbols[];
+#endif
 
 #define PRELUDE_PLUGIN_SET_PRELOADED_SYMBOLS()         \
         prelude_plugin_set_preloaded_symbols(lt_preloaded_symbols)
