@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -31,42 +31,14 @@
 #include <stdarg.h>
 
 #include "prelude-list.h"
+#include "prelude-macros.h"
 #include "prelude-inttypes.h"
+#include "prelude-linked-object.h"
 
 #ifdef __cplusplus
  extern "C" {
 #endif
 
-#ifndef __attribute__
-/* This feature is available in gcc versions 2.5 and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
-#  define __attribute__(Spec) /* empty */
-# endif
-/* The __-protected variants of `format' and `printf' attributes
-   are accepted by gcc versions 2.6.4 (effectively 2.7) and later.  */
-# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
-#  define __format__ format
-#  define __printf__ printf
-# endif
-#endif
-
-
-struct prelude_string {
-        prelude_list_t list;
-
-        int flags;
-        int refcount;
-
-        union {
-                char *rwbuf;
-                const char *robuf;
-        } data;
-
-        size_t size;
-        size_t index;
-};
-
-
 
 typedef struct prelude_string prelude_string_t;
 
@@ -119,6 +91,8 @@ int prelude_string_get_string_released(prelude_string_t *string, char **outptr);
 
 prelude_bool_t prelude_string_is_empty(const prelude_string_t *string);
 
+int prelude_string_truncate(prelude_string_t *string, size_t len);
+
 void prelude_string_clear(prelude_string_t *string);
 
 /*
@@ -128,10 +102,10 @@ int prelude_string_cat(prelude_string_t *dst, const char *str);
 int prelude_string_ncat(prelude_string_t *dst, const char *str, size_t len);
 
 int prelude_string_sprintf(prelude_string_t *string, const char *fmt, ...)
-                           __attribute__ ((__format__ (__printf__, 2, 3)));
+                           PRELUDE_FMT_PRINTF(2, 3);
 
 int prelude_string_vprintf(prelude_string_t *string, const char *fmt, va_list ap)
-                           __attribute__ ((__format__ (__printf__, 2, 0)));
+                           PRELUDE_FMT_PRINTF(2, 0);
 
 int prelude_string_compare(const prelude_string_t *str1, const prelude_string_t *str2);
 
