@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 1998-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 1998-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -90,7 +90,7 @@ void prelude_timer_destroy(prelude_timer_t *timer);
  * Wake up time that need it.
  * This function should be called every second to work properly.
  */
-void prelude_timer_wake_up(void);
+int prelude_timer_wake_up(void);
 
 /*
  *
