@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -31,9 +31,29 @@
  * This is mostly used by the libprelude detection routine in libprelude.m4
  * to make sure that the header version is consistant with the linked library.
  */
-#define LIBPRELUDE_VERSION "1.0.1"
+#define LIBPRELUDE_VERSION "3.1.0"
+
+#define LIBPRELUDE_RELEASE_LEVEL_ALPHA  0xa
+#define LIBPRELUDE_RELEASE_LEVEL_BETA   0xb
+#define LIBPRELUDE_RELEASE_LEVEL_RC     0xc
+#define LIBPRELUDE_RELEASE_LEVEL_FINAL  0xf
+
+#define LIBPRELUDE_MAJOR_VERSION  3
+#define LIBPRELUDE_MINOR_VERSION  1
+#define LIBPRELUDE_MICRO_VERSION  0
+#define LIBPRELUDE_LEVEL_VERSION  LIBPRELUDE_RELEASE_LEVEL_FINAL
+#define LIBPRELUDE_SERIAL_VERSION  0
+
+
+#define LIBPRELUDE_HEXVERSION  ((LIBPRELUDE_MAJOR_VERSION << 24) | \
+                                (LIBPRELUDE_MINOR_VERSION << 16) | \
+                                (LIBPRELUDE_MICRO_VERSION <<  8) | \
+                                (LIBPRELUDE_LEVEL_VERSION <<  4) | \
+                                (LIBPRELUDE_SERIAL_VERSION << 0))
+
 
 #include "prelude-inttypes.h"
+#include "prelude-macros.h"
 #include "common.h"
 #include "prelude-client.h"
 #include "prelude-connection.h"
@@ -57,6 +77,8 @@ int prelude_init(int *argc, char **argv);
 
 void prelude_deinit(void);
 
+int prelude_parse_version(const char *version, unsigned int *out);
+
 const char *prelude_check_version(const char *req_version);
 
 int prelude_thread_init(void *future_use);
