@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2003-2016 CS-SI. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -194,21 +194,11 @@ void prelude_hash_destroy(prelude_hash_t *hash)
 
 
 
-int prelude_hash_set(prelude_hash_t *hash, void *key, void *value)
+int prelude_hash_add(prelude_hash_t *hash, void *key, void *value)
 {
         hash_elem_t *hash_elem;
         prelude_list_t *list;
 
-        hash_elem = hash_elem_get(hash, key);
-
-        if ( hash_elem ) {
-                hash_elem_key_destroy(hash, hash_elem);
-                hash_elem_value_destroy(hash, hash_elem);
-                hash_elem->key = key;
-                hash_elem->value = value;
-                return 0;
-        }
-
         hash_elem = calloc(1, sizeof(*hash_elem));
         if ( ! hash_elem )
                 return prelude_error_from_errno(errno);
@@ -224,6 +214,23 @@ int prelude_hash_set(prelude_hash_t *hash, void *key, void *value)
 
 
 
+int prelude_hash_set(prelude_hash_t *hash, void *key, void *value)
+{
+        hash_elem_t *hash_elem = hash_elem_get(hash, key);
+
+        if ( hash_elem ) {
+                hash_elem_key_destroy(hash, hash_elem);
+                hash_elem_value_destroy(hash, hash_elem);
+                hash_elem->key = key;
+                hash_elem->value = value;
+                return 0;
+        }
+
+        return prelude_hash_add(hash, key, value);
+}
+
+
+
 void *prelude_hash_get(prelude_hash_t *hash, const void *key)
 {
         hash_elem_t *hash_elem;
