@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2001-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2001-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -549,19 +549,23 @@ int prelude_msg_forward(prelude_msg_t *msg, prelude_io_t *dst, prelude_io_t *src
 
 
 
-
 /**
- * prelude_msg_write:
+ * prelude_msg_write_r:
  * @msg: Pointer on a #prelude_msg_t object containing the message.
  * @dst: Pointer on a #prelude_io_t object to send the message to.
+ * @windex: Number of byte written to @dst
  *
- * prelude_msg_write() write the message corresponding to the @msg
+ * prelude_msg_write_r() write the message corresponding to the @msg
  * object to @dst. The message should have been created using the
  * prelude_msg_new() and prelude_msg_set() functions.
  *
+ * @windex is used to hold the number of byte written to @dst in case
+ * the write could not complete (PRELUDE_ERROR_EAGAIN). In case the write
+ * is complete, it's value is reset to 0.
+ *
  * Returns: 0 on success, or a negative value if an error occured.
  */
-int prelude_msg_write(prelude_msg_t *msg, prelude_io_t *dst)
+int prelude_msg_write_r(prelude_msg_t *msg, prelude_io_t *dst, uint32_t *windex)
 {
         ssize_t ret;
         uint32_t dlen = msg->write_index;
@@ -586,14 +590,14 @@ int prelude_msg_write(prelude_msg_t *msg, prelude_io_t *dst)
         else if ( ! msg->hdr.is_fragment )
                 dlen -= PRELUDE_MSG_HDR_SIZE;
 
-        ret = prelude_io_write(dst, msg->payload + msg->fd_write_index, dlen - msg->fd_write_index);
+        ret = prelude_io_write(dst, msg->payload + *windex, dlen - *windex);
         if ( ret < 0 )
                 return ret;
 
-        msg->fd_write_index += ret;
+        *windex += ret;
 
-        if ( msg->fd_write_index == dlen ) {
-                msg->fd_write_index = 0;
+        if ( *windex == dlen ) {
+                *windex = 0;
                 return 0;
         }
 
@@ -602,6 +606,24 @@ int prelude_msg_write(prelude_msg_t *msg, prelude_io_t *dst)
 
 
 
+/**
+ * prelude_msg_write:
+ * @msg: Pointer on a #prelude_msg_t object containing the message.
+ * @dst: Pointer on a #prelude_io_t object to send the message to.
+ *
+ * prelude_msg_write() write the message corresponding to the @msg
+ * object to @dst. The message should have been created using the
+ * prelude_msg_new() and prelude_msg_set() functions.
+ *
+ * Returns: 0 on success, or a negative value if an error occured.
+ */
+int prelude_msg_write(prelude_msg_t *msg, prelude_io_t *dst)
+{
+        return prelude_msg_write_r(msg, dst, &msg->fd_write_index);
+}
+
+
+
 
 /**
  * prelude_msg_recycle:
@@ -613,6 +635,7 @@ int prelude_msg_write(prelude_msg_t *msg, prelude_io_t *dst)
 void prelude_msg_recycle(prelude_msg_t *msg)
 {
         msg->header_index = 0;
+        msg->fd_write_index = 0;
         msg->write_index = PRELUDE_MSG_HDR_SIZE;
         msg->payload = (unsigned char *) msg + sizeof(*msg);
 
@@ -932,6 +955,23 @@ void prelude_msg_set_data(prelude_msg_t *msg, void *data)
 
 
 
+
+/**
+ * prelude_msg_get_data:
+ * @msg: Pointer on a #prelude_msg_t object.
+ *
+ * prelude_msg_get_data() allow to retrieve the data passed
+ * to the message sending callback.
+ *
+ * Returns: Pointer to the data
+ */
+void *prelude_msg_get_data(prelude_msg_t *msg)
+{
+        return msg->send_msg_data;
+}
+
+
+
 /**
  * prelude_msg_is_fragment:
  * @msg: Pointer on a #prelude_msg_t object.
