@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 1998-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 1998-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -466,7 +466,7 @@ static int plugin_load_single(prelude_list_t *head,
 
         ret = plugin_init(pe, data);
         if ( ret < 0 || ! pe->plugin ) {
-                prelude_log(PRELUDE_LOG_WARN, "%s initialization failure.\n", filename);
+                prelude_log(PRELUDE_LOG_WARN, "%s initialization failure : %s.\n", filename, prelude_strerror(ret));
                 prelude_list_del(&pe->list);
                 lt_dlclose(handle);
                 free(pe);
