@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -45,7 +45,8 @@
 #include "prelude-error.h"
 
 
-#define CHUNK_SIZE 1024
+#define BUFFER_CHUNK_SIZE 16
+#define BUFFER_GROWTH_FACTOR 1.5
 
 
 /*
@@ -59,12 +60,6 @@
 #define PRELUDE_STRING_OWN_DATA       0x2
 
 
-/*
- * Whether we can reallocate this string
- */
-#define PRELUDE_STRING_CAN_REALLOC    0x4
-
-
 
 
 #if ! defined (PRELUDE_VA_COPY)
@@ -93,6 +88,23 @@
 } while(0)
 
 
+
+struct prelude_string {
+        PRELUDE_LINKED_OBJECT;
+
+        int flags;
+        int refcount;
+
+        union {
+                char *rwbuf;
+                const char *robuf;
+        } data;
+
+        size_t size;
+        size_t index;
+};
+
+
 static int string_buf_alloc(prelude_string_t *string, size_t len)
 {
         /*
@@ -120,40 +132,55 @@ static void string_buf_copy(prelude_string_t *string, const char *buf, size_t le
 
 
 
+static int make_string_own(prelude_string_t *s, size_t len)
+{
+        char *ptr;
+
+        ptr = malloc(len);
+        if ( ! ptr )
+                return prelude_error_from_errno(errno);
+
+        if ( s->data.robuf )
+                memcpy(ptr, s->data.robuf, s->index + 1);
+
+        s->size = len;
+        s->data.rwbuf = ptr;
+        s->flags |= PRELUDE_STRING_OWN_DATA;
+
+        return 0;
+}
+
+
+
 static int allocate_more_chunk_if_needed(prelude_string_t *s, size_t needed_len)
 {
+        int ret;
         char *ptr;
         size_t len;
 
-        if ( needed_len )
-                len = MAX(needed_len - (s->size - s->index), CHUNK_SIZE);
+        if ( ! needed_len )
+                len = BUFFER_CHUNK_SIZE;
         else
-                len = CHUNK_SIZE;
+                len = MAX(needed_len - (s->size - s->index), s->size * BUFFER_GROWTH_FACTOR);
 
         if ( s->size + len < s->size )
                 return prelude_error(PRELUDE_ERROR_INVAL_LENGTH);
 
-        if ( s->flags & PRELUDE_STRING_CAN_REALLOC ) {
-
+        if ( s->flags & PRELUDE_STRING_OWN_DATA ) {
                 ptr = _prelude_realloc(s->data.rwbuf, s->size + len);
                 if ( ! ptr )
                         return prelude_error_from_errno(errno);
+
+                s->size += len;
+                s->data.rwbuf = ptr;
         }
 
         else {
-                ptr = malloc(s->size + len);
-                if ( ! ptr )
-                        return prelude_error_from_errno(errno);
-
-                if ( s->data.robuf )
-                        memcpy(ptr, s->data.robuf, s->index + 1);
-
-                s->flags |= PRELUDE_STRING_CAN_REALLOC|PRELUDE_STRING_OWN_DATA;
+                ret = make_string_own(s, s->size + len);
+                if ( ret < 0 )
+                        return ret;
         }
 
-        s->size += len;
-        s->data.rwbuf = ptr;
-
         return 0;
 }
 
@@ -174,7 +201,7 @@ int prelude_string_new(prelude_string_t **string)
                 return prelude_error_from_errno(errno);
 
         (*string)->refcount = 1;
-        prelude_list_init(&(*string)->list);
+        prelude_list_init(&(*string)->_list);
         (*string)->flags = PRELUDE_STRING_OWN_STRUCTURE;
 
         return 0;
@@ -229,7 +256,7 @@ int prelude_string_new_dup_fast(prelude_string_t **string, const char *str, size
                 return ret;
 
         string_buf_copy(*string, str, len);
-        (*string)->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
+        (*string)->flags |= PRELUDE_STRING_OWN_DATA;
 
         return 0;
 }
@@ -282,7 +309,7 @@ int prelude_string_new_nodup_fast(prelude_string_t **string, char *str, size_t l
         (*string)->index = len;
         (*string)->size = len + 1;
         (*string)->data.rwbuf = str;
-        (*string)->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
+        (*string)->flags |= PRELUDE_STRING_OWN_DATA;
 
         return 0;
 }
@@ -383,7 +410,7 @@ int prelude_string_set_dup_fast(prelude_string_t *string, const char *buf, size_
                 return ret;
 
         string_buf_copy(string, buf, len);
-        string->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
+        string->flags |= PRELUDE_STRING_OWN_DATA;
 
         return 0;
 }
@@ -432,7 +459,7 @@ int prelude_string_set_nodup_fast(prelude_string_t *string, char *buf, size_t le
         string->size = len + 1;
         string->data.rwbuf = buf;
 
-        string->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
+        string->flags |= PRELUDE_STRING_OWN_DATA;
 
         return 0;
 }
@@ -483,7 +510,7 @@ int prelude_string_set_ref_fast(prelude_string_t *string, const char *buf, size_
         string->size = len + 1;
         string->data.robuf = buf;
 
-        string->flags &= ~(PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC);
+        string->flags &= ~PRELUDE_STRING_OWN_DATA;
 
         return 0;
 }
@@ -531,7 +558,7 @@ int prelude_string_copy_ref(const prelude_string_t *src, prelude_string_t *dst)
         dst->size = src->size;
         dst->index = src->index;
         dst->data.robuf = src->data.robuf;
-        dst->flags &= ~(PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC);
+        dst->flags &= ~PRELUDE_STRING_OWN_DATA;
 
         return 0;
 }
@@ -557,7 +584,7 @@ int prelude_string_copy_dup(const prelude_string_t *src, prelude_string_t *dst)
 
         dst->size = src->size;
         dst->index = src->index;
-        dst->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
+        dst->flags |= PRELUDE_STRING_OWN_DATA;
 
         if ( src->size ) {
                 dst->data.rwbuf = malloc(src->size);
@@ -594,7 +621,7 @@ int prelude_string_clone(const prelude_string_t *src, prelude_string_t **dst)
 
         (*dst)->size = src->size;
         (*dst)->index = src->index;
-        (*dst)->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
+        (*dst)->flags |= PRELUDE_STRING_OWN_DATA;
 
         if ( src->size ) {
                 (*dst)->data.rwbuf = malloc(src->size);
@@ -683,7 +710,7 @@ int prelude_string_get_string_released(prelude_string_t *string, char **outptr)
         if ( ! string->index )
                 return 0;
 
-        if ( ! (string->flags & PRELUDE_STRING_CAN_REALLOC) ) {
+        if ( ! (string->flags & PRELUDE_STRING_OWN_DATA) ) {
                 *outptr = strdup(string->data.robuf);
                 return (*outptr) ? 0 : prelude_error_from_errno(errno);
         }
@@ -757,8 +784,8 @@ void prelude_string_destroy(prelude_string_t *string)
         if ( --string->refcount )
                 return;
 
-        if ( ! prelude_list_is_empty(&string->list) )
-                prelude_list_del_init(&string->list);
+        if ( ! prelude_list_is_empty(&string->_list) )
+                prelude_list_del_init(&string->_list);
 
         prelude_string_destroy_internal(string);
 
@@ -789,7 +816,7 @@ int prelude_string_vprintf(prelude_string_t *string, const char *fmt, va_list ap
         prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(fmt, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        if ( ! (string->flags & PRELUDE_STRING_CAN_REALLOC) ) {
+        if ( ! (string->flags & PRELUDE_STRING_OWN_DATA) ) {
 
                 ret = allocate_more_chunk_if_needed(string, 0);
                 if ( ret < 0 )
@@ -876,7 +903,7 @@ int prelude_string_ncat(prelude_string_t *dst, const char *str, size_t len)
         prelude_return_val_if_fail(dst, prelude_error(PRELUDE_ERROR_ASSERTION));
         prelude_return_val_if_fail(str, prelude_error(PRELUDE_ERROR_ASSERTION));
 
-        if ( dst->flags & PRELUDE_STRING_CAN_REALLOC && len < (dst->size - dst->index) ) {
+        if ( dst->flags & PRELUDE_STRING_OWN_DATA && len < (dst->size - dst->index) ) {
 
                 memcpy(dst->data.rwbuf + dst->index, str, len);
 
@@ -920,6 +947,37 @@ int prelude_string_cat(prelude_string_t *dst, const char *str)
 
 
 
+/**
+ * prelude_string_truncate:
+ * @string: Pointer to a #prelude_string_t object.
+ * @len: New @string size
+ *
+ * Truncate @string content.
+ */
+int prelude_string_truncate(prelude_string_t *string, size_t len)
+{
+        int ret;
+
+        prelude_return_val_if_fail(string, prelude_error(PRELUDE_ERROR_ASSERTION));
+        prelude_return_val_if_fail(len <= string->index, prelude_error(PRELUDE_ERROR_ASSERTION));
+
+        if ( len && ! (string->flags & PRELUDE_STRING_OWN_DATA) ) {
+                ret = make_string_own(string, 0);
+                if ( ret < 0 )
+                        return ret;
+        }
+
+        if ( string->data.rwbuf )
+                *(string->data.rwbuf + len) = '\0';
+        else
+                string->data.robuf = NULL;
+
+        string->index = len;
+        return 0;
+}
+
+
+
 /**
  * prelude_string_clear:
  * @string: Pointer to a #prelude_string_t object.
@@ -928,23 +986,19 @@ int prelude_string_cat(prelude_string_t *dst, const char *str)
  */
 void prelude_string_clear(prelude_string_t *string)
 {
-        prelude_return_if_fail(string);
-
-        string->index = 0;
-
-        if ( string->data.rwbuf && string->flags & PRELUDE_STRING_OWN_DATA )
-                *(string->data.rwbuf) = '\0';
-
-        else string->data.rwbuf = NULL;
+        prelude_string_truncate(string, 0);
 }
 
 
 
+
 /**
  * prelude_string_compare:
  * @str1: Pointer to a #prelude_string_t object to compare with @str2.
  * @str2: Pointer to a #prelude_string_t object to compare with @str1.
  *
+ * Compare @str1 and @str2.
+ *
  * Returns: 0 if @str1 and @str2 value are equal, a negative value otherwise.
  */
 int prelude_string_compare(const prelude_string_t *str1, const prelude_string_t *str2)
@@ -961,5 +1015,5 @@ int prelude_string_compare(const prelude_string_t *str1, const prelude_string_t
         if ( str1->index == 0 )
                 return 0;
 
-        return strcmp(str1->data.robuf, str2->data.robuf);
+        return strcmp(str1->data.robuf, str2->data.robuf) == 0 ? 0 : -1;
 }
