@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004-2012 CS-SI. All Rights Reserved.
+* Copyright (C) 2004-2016 CS-SI. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -15,9 +15,9 @@
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
-* You should have received a copy of the GNU General Public License
-* along with this program; see the file COPYING.  If not, write to
-* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
+* You should have received a copy of the GNU General Public License along
+* with this program; if not, write to the Free Software Foundation, Inc.,
+* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 *****/
 
@@ -270,6 +270,67 @@ void prelude_deinit(void)
 
 
 
+static int levelstr_to_int(const char *wanted)
+{
+        int i;
+        struct {
+                int level;
+                const char *level_str;
+        } tbl[] = {
+                { LIBPRELUDE_RELEASE_LEVEL_ALPHA, "alpha" },
+                { LIBPRELUDE_RELEASE_LEVEL_BETA, "beta"   },
+                { LIBPRELUDE_RELEASE_LEVEL_RC, "rc"       }
+        };
+
+        for ( i = 0; i < sizeof(tbl) / sizeof(*tbl); i++ ) {
+                if ( strcmp(wanted, tbl[i].level_str) == 0 )
+                        return tbl[i].level;
+        }
+
+        return -1;
+}
+
+
+
+/**
+ * prelude_parse_version:
+ * @version: A version string.
+ * @out: Where to store the parsed version
+ *
+ * Parse version to an integer, and return it in @out.
+ * Accepted format are:
+ *
+ * major.minor.micro.patchlevel
+ * the following special level string are supported : alpha, beta, rc
+ *
+ * For example: 1.1.1rc1
+ *
+ * Returns: The 0 on success, a negative value in case of error.
+ */
+int prelude_parse_version(const char *version, unsigned int *out)
+{
+        int ret;
+        char levels[6] = { 0 };
+        int major = 0, minor = 0, micro = 0, level = 0, patch = 0;
+
+        ret = sscanf(version, "%d.%d.%d%5[^0-9]%d", &major, &minor, &micro, levels, &patch);
+        if ( ret <= 0 )
+                return prelude_error_verbose(PRELUDE_ERROR_GENERIC, "version formatting error with '%s'", version);
+
+        if ( *levels == 0 || *levels == '.' )
+                level = LIBPRELUDE_RELEASE_LEVEL_FINAL;
+        else {
+                level = levelstr_to_int(levels);
+                if ( level < 0 )
+                        return level;
+        }
+
+        *out = (major << 24) | (minor << 16) | (micro << 8) | (level << 4) | (patch << 0);
+        return 0;
+}
+
+
+
 /**
  * prelude_check_version:
  * @req_version: The minimum acceptable version number.
@@ -284,28 +345,16 @@ void prelude_deinit(void)
 const char *prelude_check_version(const char *req_version)
 {
         int ret;
-        int major, minor, micro, patch = 0;
-        int rq_major, rq_minor, rq_micro, rq_patch = 0;
+        unsigned int hexversion;
 
         if ( ! req_version )
                 return VERSION;
 
-        ret = sscanf(VERSION, "%d.%d.%d.%d", &major, &minor, &micro, &patch);
-        if ( ret < 3 )
+        ret = prelude_parse_version(req_version, &hexversion);
+        if ( ret < 0 )
                 return NULL;
 
-        ret = sscanf(req_version, "%d.%d.%d.%d", &rq_major, &rq_minor, &rq_micro, &rq_patch);
-        if ( ret < 3 )
-                return NULL;
-
-        if ( major > rq_major
-             || (major == rq_major && minor > rq_minor)
-             || (major == rq_major && minor == rq_minor && micro > rq_micro)
-             || (major == rq_major && minor == rq_minor && micro == rq_micro && patch >= rq_patch) ) {
-                return VERSION;
-        }
-
-        return NULL;
+        return (hexversion <= LIBPRELUDE_HEXVERSION) ? VERSION : NULL;
 }
 
 
